# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Example how to add additional text to a SVG QR code.
"""
import io
import re
from xml.sax.saxutils import escape
import segno
from segno.writers import _color_to_webcolor as webcolor


def qr_with_text(qrcode: segno.QRCode, *, text: str = None, font_size: int = 12,
                 font_color=False, line_spacing: int = None, scale: int = 1,
                 border: int = None, dark='#000', light='#fff', finder_dark=False,
                 finder_light=False, data_dark=False, data_light=False,
                 version_dark=False, version_light=False, format_dark=False,
                 format_light=False, alignment_dark=False, alignment_light=False,
                 timing_dark=False, timing_light=False, separator=False,
                 dark_module=False, quiet_zone=False, **kw) -> io.BytesIO:
    """\
    Adds a text (i.e. the content of the QR code) to the QR code.

    See `segno.QRCode.save <https://segno.readthedocs.io/en/latest/api.html#segno.QRCode.save>`_
    and `SVG <https://segno.readthedocs.io/en/latest/api.html#svg>`_ for a detailed
    description of the parameters.

    :param segno.QRCode qrcode: The QR code.
    :param str text: The text to add to the QR code.
    :param int font_size: Font size.
    :param font_color: See dark for valid values.
    :param int line_spacing: Spacing between the lines. If not provided, it
            is the half of the font size.
    :param scale: The scale.
    :param int border: Number indicating the size of the quiet zone.
            If set to ``None`` (default), the recommended border size
            will be used (``4`` for QR Codes, ``2`` for Micro QR Codes).
    :param dark: Color of the dark modules (default: black). The
            color can be provided as ``(R, G, B)`` tuple, as hexadecimal
            format (``#RGB``, ``#RRGGBB`` ``RRGGBBAA``), or web color
            name (i.e. ``red``).
    :param light: Color of the light modules (default: white).
            See `color` for valid values. If light is set to ``None`` the
            light modules will be transparent.
    :param finder_dark: Color of the dark finder modules (default: same as ``dark``)
    :param finder_light: Color of the light finder modules (default: same as ``light``)
    :param data_dark: Color of the dark data modules (default: same as ``dark``)
    :param data_light: Color of the light data modules (default: same as ``light``)
    :param version_dark: Color of the dark version modules (default: same as ``dark``)
    :param version_light: Color of the light version modules (default: same as ``light``)
    :param format_dark: Color of the dark format modules (default: same as ``dark``)
    :param format_light: Color of the light format modules (default: same as ``light``)
    :param alignment_dark: Color of the dark alignment modules (default: same as ``dark``)
    :param alignment_light: Color of the light alignment modules (default: same as ``light``)
    :param timing_dark: Color of the dark timing pattern modules (default: same as ``dark``)
    :param timing_light: Color of the light timing pattern modules (default: same as ``light``)
    :param separator: Color of the separator (default: same as ``light``)
    :param dark_module: Color of the dark module (default: same as ``dark``)
    :param quiet_zone: Color of the quiet zone modules (default: same as ``light``)
    """
    def calc_font_size(txt) -> int:
        return round(len(txt) * font_size / 1.5), font_size

    svg = io.BytesIO()
    qrcode.save(svg, kind='svg', scale=scale, border=border, dark=dark,
                light=light, finder_dark=finder_dark, finder_light=finder_light,
                data_dark=data_dark, data_light=data_light, version_dark=version_dark,
                version_light=version_light, format_dark=format_dark,
                format_light=format_light, alignment_dark=alignment_dark,
                alignment_light=alignment_light, timing_dark=timing_dark,
                timing_light=timing_light, separator=separator,
                dark_module=dark_module, quiet_zone=quiet_zone, **kw)
    svg.seek(0)
    if text is None:
        text = kw.get('title')
    if text is None:
        # That was easy, nothing to do, just return the SVG
        return svg
    encoding = kw.get('encoding', 'utf-8')
    qr_width, qr_height = qrcode.symbol_size(scale, border)
    width, height = qr_width, qr_height
    text_buff = io.BytesIO()
    write = text_buff.write
    write_str = lambda s: write(s.encode(encoding))
    border = border if border is not None else qrcode.default_border_size
    border_offset = scale * border
    font_color = webcolor(font_color or dark)
    write_str('<text y="{}" font-size="{}" font-family="mono" fill="{}">'
              .format(qr_height, font_size, font_color))
    line_spacing = line_spacing or font_size // 2
    for line in text.splitlines():
        write_str('<tspan x="{}" dy="{}">'.format(border_offset, font_size + line_spacing))
        write_str(escape(line))
        write(b'</tspan>')
        fw, fh = calc_font_size(line)
        if fw > width:
            width = fw
        height += fh + line_spacing
    height += line_spacing
    write(b'</text></svg>')
    # Add the text to the SVG generated by Segno.
    # The </svg> is part of the text_buff
    res = re.sub(b'</svg>', text_buff.getvalue(), svg.getvalue())
    res = re.sub('width="{}"'.format(qr_width).encode(encoding),
                 'width="{}"'.format(width).encode(encoding), res)
    res = re.sub('height="{}"'.format(qr_height).encode(encoding),
                 'height="{}"'.format(height).encode(encoding), res)
    svg = io.BytesIO(res)
    svg.seek(0)
    return svg


if __name__ == '__main__':
    content = 'If the rain comes\n' \
              'They run and hide their heads\n' \
              'They might as well be dead\n' \
              'If the rain comes\n' \
              'If the rain comes'
    qr = segno.make(content)
    svg = qr_with_text(qr, text=content, scale=6, dark='darkblue', light='#eee',
                       data_dark='steelblue').getvalue()
    with open('rain-1.svg', 'wb') as f:
        f.write(svg)
    svg = qr_with_text(qr, text=content, scale=6).getvalue()
    with open('rain-2.svg', 'wb') as f:
        f.write(svg)
    svg = qr_with_text(qr, text=content, scale=6, font_size=32).getvalue()
    with open('rain-3.svg', 'wb') as f:
        f.write(svg)
    svg = qr_with_text(qr, text=content).getvalue()
    with open('rain-4.svg', 'wb') as f:
        f.write(svg)
    svg = qr_with_text(qr, text=content, border=0, scale=3).getvalue()
    with open('rain-5.svg', 'wb') as f:
        f.write(svg)
    svg = qr_with_text(qr, title='Rain', scale=3, font_color='darkred').getvalue()
    with open('rain-6.svg', 'wb') as f:
        f.write(svg)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue #84.
<https://github.com/heuer/segno/issues/84>
"""
from __future__ import unicode_literals, absolute_import
import os
import tempfile
from segno import cli
import pytest


def test_issue_84_default_encoding():
    with open(os.path.join(os.path.dirname(__file__), 'issue-84',
                           'issue-84-iso-8859-1.txt'), 'r') as f:
        expected = f.read()
    f = tempfile.NamedTemporaryFile('w', suffix='.txt', delete=False)
    f.close()
    try:
        cli.main(['-o', f.name, 'Müller'])
        with open(f.name, 'r') as f:
            result = f.read()
        assert expected == result
        # Explicit but default encoding
        cli.main(['-o', f.name, '--encoding', 'iso-8859-1', 'Müller'])
        with open(f.name, 'r') as f:
            result = f.read()
        assert expected == result
        # Explicit but default encoding
        cli.main(['-o', f.name, '--encoding', 'latin1', 'Müller'])
        with open(f.name, 'r') as f:
            result = f.read()
        assert expected == result
    finally:
        os.unlink(f.name)


def test_issue_84_utf8():
    with open(os.path.join(os.path.dirname(__file__), 'issue-84',
                           'issue-84-utf-8.txt'), 'r') as f:
        expected = f.read()
    f = tempfile.NamedTemporaryFile('w', suffix='.txt', delete=False)
    f.close()
    try:
        cli.main(['-o', f.name, '--encoding', 'utf-8', 'Müller'])
        with open(f.name, 'r') as f:
            result = f.read()
        assert expected == result
        cli.main(['-o', f.name, '--encoding', 'UTf-8', 'Müller'])
        with open(f.name, 'r') as f:
            result = f.read()
        assert expected == result
    finally:
        os.unlink(f.name)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- encoding: utf-8 -*-
"""\
Some benchmarks running against QR Code generators
"""
from __future__ import print_function
import sys
import os
import csv
import timeit
import segno
try:
    import qrcode
    from qrcode import QRCode, ERROR_CORRECT_M, ERROR_CORRECT_Q, ERROR_CORRECT_H
    from qrcode.image.svg import SvgImage, SvgPathImage
except ImportError:
    qrcode = None
try:
    import qrcodegen
    from qrcodegen import QrCode, QrSegment
    qrcodegen_make_segment = QrSegment.make_segments
    qrcodegen_error_m = QrCode.Ecc.MEDIUM
    qrcodegen_error_q = QrCode.Ecc.QUARTILE
    qrcodegen_error_h = QrCode.Ecc.HIGH
except ImportError:
    qrcodegen = None
try:
    import pyqrcode
except ImportError:
    pyqrcode = None

if qrcode:
    def create_qrcode(data='QR Code Symbol'):
        """qrcode create 1-M"""
        qr = QRCode(error_correction=ERROR_CORRECT_M)
        qr.add_data(data, optimize=False)
        qr.make()

    def create7q_qrcode(data='QR Code Symbol'):
        """qrcode create 7-Q"""
        qr = QRCode(error_correction=ERROR_CORRECT_Q, version=7)
        qr.add_data(data, optimize=False)
        qr.make()

    def create30h_qrcode(data='QR Code Symbol'):
        """qrcode create 30-H"""
        qr = QRCode(error_correction=ERROR_CORRECT_H, version=30)
        qr.add_data(data, optimize=False)
        qr.make()

    def svg_qrcode_path(data='QR Code Symbol'):
        """qrcode SVG path"""
        qr = QRCode(error_correction=ERROR_CORRECT_M, box_size=10,
                    image_factory=SvgPathImage)
        qr.add_data(data, optimize=False)
        qr.make_image().save(os.path.join(_output_dir(), 'qrcode_path_%s.svg' % data))

    def svg_qrcode_rects(data='QR Code Symbol'):
        """qrcode SVG rects"""
        qr = QRCode(error_correction=ERROR_CORRECT_M, box_size=10,
                    image_factory=SvgImage)
        qr.add_data(data, optimize=False)
        qr.make_image().save(os.path.join(_output_dir(), 'qrcode_rects_%s.svg' % data))

    def png_qrcode(data='QR Code Symbol'):
        """qrcode PNG 1-M"""
        qr = QRCode(error_correction=ERROR_CORRECT_M, box_size=10)
        qr.add_data(data, optimize=False)
        qr.make_image().save(os.path.join(_output_dir(), 'qrcode_%s.png' % data))


if pyqrcode:
    def create_pyqrcode(data='QR Code Symbol'):
        """PyQRCode create 1-M"""
        pyqrcode.create(data, error='m')

    def create7q_pyqrcode(data='QR Code Symbol'):
        """PyQRCode create 7-Q"""
        pyqrcode.create(data, error='q', version=7)

    def create30h_pyqrcode(data='QR Code Symbol'):
        """PyQRCode create 30-H"""
        pyqrcode.create(data, error='h', version=30)

    def svg_pyqrcode(data='QR Code Symbol'):
        """PyQRCode SVG"""
        pyqrcode.create(data, error='m').svg(os.path.join(_output_dir(), 'pyqrcode_%s.svg' % data), scale=10)

    def png_pyqrcode(data='QR Code Symbol'):
        """PyQRCode PNG 1-M"""
        pyqrcode.create(data, error='m').png(os.path.join(_output_dir(), 'pyqrcode_%s.png' % data), scale=10)

if qrcodegen:
    def create_qrcodegen(data='QR Code Symbol'):
        """qrcodegen create 1-M"""
        QrCode.encode_segments(qrcodegen_make_segment(data),
                               ecl=qrcodegen_error_m,
                               boostecl=False)

    def create7q_qrcodegen(data='QR Code Symbol'):
        """qrcodegen create 7-Q"""
        QrCode.encode_segments(qrcodegen_make_segment(data),
                               ecl=qrcodegen_error_q,
                               minversion=7,
                               maxversion=7,
                               boostecl=False)

    def create30h_qrcodegen(data='QR Code Symbol'):
        """qrcodegen create 30-H"""
        QrCode.encode_segments(qrcodegen_make_segment(data),
                               ecl=qrcodegen_error_h,
                               minversion=30,
                               maxversion=30,
                               boostecl=False)


def _output_dir():
    return os.path.join(os.path.abspath(os.path.dirname(__file__)), 'out')


def create_segno(data='QR Code Symbol'):
    """Segno create 1-M"""
    segno.make_qr(data, error='m', boost_error=False)


def create7q_segno(data='QR Code Symbol'):
    """Segno create 7-Q"""
    segno.make_qr(data, error='q', version=7, boost_error=False)


def create30h_segno(data='QR Code Symbol'):
    """Segno create 30-H"""
    segno.make_qr(data, error='h', version=30, boost_error=False)


def svg_segno(data='QR Code Symbol'):
    """Segno SVG"""
    segno.make_qr(data, error='m', boost_error=False).save(os.path.join(_output_dir(), 'segno_%s.svg' % data), scale=10)


def png_segno(data='QR Code Symbol'):
    """Segno PNG 1-M"""
    segno.make_qr(data, error='m', boost_error=False).save(os.path.join(_output_dir(), 'segno_%s.png' % data), scale=10)


def run_create_tests(which=None, number=200, table=None):
    tests = ('create_pyqrcode',
             'create_pyqrcodeng',
             'create_qrcodegen',
             'create_qrcode',
             'create_segno',)
    if which:
        tests = filter(lambda n: n[len('create_'):] in which, tests)
    _run_tests(tests, number, table)


def run_create7q_tests(which=None, number=200, table=None):
    tests = ('create7q_pyqrcode',
             'create7q_pyqrcodeng',
             'create7q_qrcodegen',
             'create7q_qrcode',
             'create7q_segno',)
    if which:
        tests = filter(lambda n: n[len('create7q_'):] in which, tests)
    _run_tests(tests, number, table)


def run_create30h_tests(which=None, number=200, table=None):
    tests = ('create30h_pyqrcode',
             'create30h_pyqrcodeng',
             'create30h_qrcodegen',
             'create30h_qrcode',
             'create30h_segno',)
    if which:
        tests = filter(lambda n: n[len('createbig_'):] in which, tests)
    _run_tests(tests, number, table)


def run_svg_tests(which=None, number=200, table=None):
    tests = ('svg_pyqrcode',
             'svg_pyqrcodeng',
             'svg_qrcode_path', 'svg_qrcode_rects',
             'svg_segno',)

    if which:
        tests = filter(lambda n: n[len('svg_'):] in which, tests)
    _run_tests(tests, number, table)


def run_png_tests(which=None, number=200, table=None):
    tests = ('png_pyqrcode',
             'png_pyqrcodeng',
             'png_qrcode',
             'png_segno',)
    if which:
        tests = filter(lambda n: n[len('png_'):] in which, tests)
    _run_tests(tests, number, table)


def _run_tests(tests, number, table=None):
    # Code taken from <https://genshi.edgewall.org/browser/trunk/examples/bench/bigtable.py>
    # Author: Jonas Borgström <jonas@edgewall.com>
    # License: BSD (I'd think since Genshi uses BSD as well)
    for test in [t for t in tests if hasattr(sys.modules[__name__], t)]:
        t = timeit.Timer(setup='from __main__ import %s;' % test, stmt='%s()' % test)
        time = t.timeit(number=number) / number
        if time < 0.00001:
            result = '   (not installed?)'
        else:
            result = '%16.2f ms' % (1000 * time)
        name = getattr(sys.modules[__name__], test).__doc__
        print('%-35s %s' % (name, result))
        if table is not None:
            table.append((name, '%.2f' % (1000 * time)))


if __name__ == '__main__':
    table = []
    run_create_tests(table=table)
    run_create7q_tests(table=table)
    run_create30h_tests(table=table)
    run_svg_tests(table=table)
    run_png_tests(table=table)
    with open(os.path.join(_output_dir(), 'results.csv'), 'w') as f:
        writer = csv.writer(f)
        writer.writerows(table)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #16.
<https://github.com/heuer/segno/issues/16>
"""
from __future__ import unicode_literals, absolute_import
import segno


def test_boost_error_automatic():
    qr = segno.make_qr('ABCDEF')
    assert '1-H' == qr.designator


def test_boost_error_automatic_disabled():
    qr = segno.make_qr('ABCDEF', boost_error=False)
    assert '1-L' == qr.designator


def test_boost_error_automatic_arg_error():
    qr = segno.make_qr('ABCDEF', error='l')
    assert '1-H' == qr.designator


def test_boost_error_disabled_arg_error():
    qr = segno.make_qr('ABCDEF', error='l', boost_error=False)
    assert '1-L' == qr.designator


def test_boost_error_m1():
    qr = segno.make('01234')
    assert qr.is_micro
    assert 'M1' == qr.version
    assert qr.error is None


def test_boost_error_micro():
    qr = segno.make('A', error='l')
    assert qr.is_micro
    assert 'M2' == qr.version
    assert 'M' == qr.error


def test_boost_error_micro_boost_disabled():
    qr = segno.make('A', error='l', boost_error=False)
    assert qr.is_micro
    assert 'M2' == qr.version
    assert 'L' == qr.error


def test_boost_error_m3():
    qr = segno.make('A', error='l', version='M3')
    assert qr.is_micro
    assert 'M3' == qr.version
    assert 'M' == qr.error


def test_boost_error_m3_boost_disabled():
    qr = segno.make('A', error='l', version='M3', boost_error=False)
    assert qr.is_micro
    assert 'M3' == qr.version
    assert 'L' == qr.error


def test_boost_error_m4():
    qr = segno.make('A', error='l', version='M4')
    assert qr.is_micro
    assert 'M4' == qr.version
    assert 'Q' == qr.error


def test_boost_error_m4_boost_disabled():
    qr = segno.make('A', error='l', version='M4', boost_error=False)
    assert qr.is_micro
    assert 'M4' == qr.version
    assert 'L' == qr.error


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue 34
<https://github.com/heuer/segno/issues/34>
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno
try:
    from .tutils import read_matrix
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from tutils import read_matrix


def test_m3_wikipedia():
    qr = segno.make('Wikipedia', version='m3')
    assert 'M3-L' == qr.designator
    ref_matrix = read_matrix('issue-33-m3-l-wikipedia')
    assert ref_matrix == qr.matrix


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
XPM output related tests.
"""
from __future__ import absolute_import, unicode_literals
import io
import re
import segno


def test_write_xpm_indicator():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='xpm')
    expected = '/* XPM */\n'
    val = out.getvalue()
    assert expected == val[:len(expected)]


def test_dark_transparent():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='xpm', dark=None, light='white')
    expected = '/* XPM */\n'
    val = out.getvalue()
    assert expected == val[:len(expected)]
    assert re.search(r'^"X c None"', val, flags=re.MULTILINE)


def test_light_transparent():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='xpm', light=None)
    expected = '/* XPM */\n'
    val = out.getvalue()
    assert expected == val[:len(expected)]
    assert re.search(r'^"  c None"', val, flags=re.MULTILINE)


_DATA_PATTERN = re.compile(r'{([^}]+)};')


def _img_data(s):
    m = _DATA_PATTERN.search(s)
    data = m.group(1).replace('"', '').replace('\n', '').split(',')
    return data


def test_write_xpm_width_height():
    scale = 5
    border = 2
    qr = segno.make_qr('test')
    width, height = qr.symbol_size(scale=scale, border=border)
    out = io.StringIO()
    qr.save(out, kind='xpm', border=border, scale=scale)
    img_data = _img_data(out.getvalue())[0]
    assert img_data.startswith('{0} {1}'.format(width, height))


def xpm_as_matrix(buff, border):
    """\
    Returns the XPM QR code as list of [0, 1] lists.

    :param io.StringIO buff: Buffer to read the matrix from.
    """
    res = []
    img_data = _img_data(buff.getvalue())
    height = int(img_data[0].split(' ')[0])
    img_data = img_data[3:]
    for i, row in enumerate(img_data):
        if i < border:
            continue
        if i >= height - border:
            break
        r = row[border:-border] if border else row
        res.append([(1 if b == 'X' else 0) for b in r])
    return res


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Example Flask app to show different possibilities to embed Segno.
"""
import io
from flask import Flask, render_template, request, send_file, abort
import segno

app = Flask(__name__)

@app.route('/')
def home():
    qr = segno.make('The Continuing Story of Bungalow Bill')
    return render_template('example.html', qr=qr)


@app.route('/qr-svg/')
def qrcode_svg():
    data = request.args.get('data')
    if data not in ('Savoy Truffle', 'Rocky Raccoon'):
        return abort(404)
    buff = io.BytesIO()
    segno.make(data, micro=False).save(buff, kind='svg', scale=4)
    buff.seek(0)
    return send_file(buff, mimetype='image/svg+xml')


@app.route('/qr-png/')
def qrcode_png():
    data = request.args.get('data')
    if data not in ('Savoy Truffle', 'Rocky Raccoon'):
        return abort(404)
    buff = io.BytesIO()
    segno.make(data, micro=False) \
        .save(buff, kind='png', scale=4, dark='darkblue', data_dark='#474747',
              light='#efefef')
    buff.seek(0)
    return send_file(buff, mimetype='image/png')


if __name__ == '__main__':
    app.run(debug=True, port=5673)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue 35
<https://github.com/heuer/segno/issues/35>
"""
from __future__ import absolute_import, unicode_literals
import pytest
from segno import consts
from segno import encoder


@pytest.mark.parametrize('version, mask', [(consts.VERSION_M1, 4),
                                           (1, 8),
                                           (1, -1),
                                           (consts.VERSION_M2, -1)])
def test_normalize_mask_illegal(version, mask):
    with pytest.raises(ValueError) as ex:
        encoder.normalize_mask(mask, version < 1)
    assert 'Invalid data mask' in str(ex.value)


@pytest.mark.parametrize('version, mask', [(consts.VERSION_M1, 'A'),
                                           (1, 'B')])
def test_normalize_mask_not_int(version, mask):
    with pytest.raises(ValueError) as ex:
        encoder.normalize_mask(mask, version < 1)
    assert 'Invalid data mask' in str(ex.value)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
PAM related tests.
"""
from __future__ import unicode_literals, absolute_import
import re
import io
import pytest
import segno
try:
    range = xrange  # noqa
except NameError:
    pass


def test_invalid_color():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='pam', dark=None)


def test_bw():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pam')
    assert b'BLACKANDWHITE' in out.getvalue()


def test_grayscale():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pam', light=None)
    assert b'GRAYSCALE_ALPHA' in out.getvalue()


def test_rgb():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pam', dark='red')
    assert b'RGB' in out.getvalue()
    assert b'RGB_ALPHA' not in out.getvalue()


def test_rgba():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pam', dark='red', light=None)
    assert b'RGB' in out.getvalue()
    assert b'RGB_ALPHA' in out.getvalue()


_size = re.compile(br'^WIDTH\s+([0-9]+)$').match


def _image_data(buff):
    """\
    Returns the image data and the size of the matrix.
    """
    seen_size = False
    size = 0
    code = buff.getvalue().splitlines()
    code_iter = iter(code)
    for line in code_iter:
        if line.startswith(b'ENDHDR'):
            break
        if seen_size:
            continue
        m = _size(line)
        if m:
            size = int(m.group(1))
            seen_size = True
    return next(code_iter), size


def pam_bw_as_matrix(buff, border):
    """\
    Returns the QR code as list of [0, 1] lists.

    :param io.BytesIO buff: Buffer to read the matrix from.
    :param int border: The QR code border
    """
    res = []
    data, size = _image_data(buff)
    for i, offset in enumerate(range(0, len(data), size)):
        if i < border:
            continue
        if i >= size - border:
            break
        row_data = bytearray(data[offset + border:offset + size - border])
        # Invert bytes since PAM uses 0x0 = black, 0x1 = white
        res.append([b ^ 0x1 for b in row_data])
    return res


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #95.
<https://github.com/heuer/segno/issues/95>

See <https://github.com/zxing/zxing/wiki/Barcode-Contents#wi-fi-network-config-android-ios-11>

This module borrows a lot of code from
<https://github.com/zxing/zxing/blob/master/core/src/test/java/com/google/zxing/client/result/WifiParsedResultTestCase.java#L59>  # noqa: E501
copyrighted by ZXing authors:

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
"""
from __future__ import unicode_literals, absolute_import
import io
import pytest
import segno
from segno import helpers
_qr_decoder_available = False
try:
    FileNotFoundError
except NameError:  # Py2
    FileNotFoundError = OSError
try:
    from pyzbar.pyzbar import decode as zbardecode
    _qr_decoder_available = True
except (ImportError, FileNotFoundError):  # The latter may occur under Windows
    pass


def qr_to_bytes(qrcode, scale):
    if qrcode.is_micro:
        raise Exception('zbar cannot decode Micro QR codes')
    buff = io.BytesIO()
    for row in qrcode.matrix_iter(scale=scale):
        buff.write(bytearray(0x0 if b else 0xff for b in row))
    return buff.getvalue()


def decode(data):
    scale = 3
    qrcode = segno.make(data, micro=False)
    width, height = qrcode.symbol_size(scale=scale)
    qr_bytes = qr_to_bytes(qrcode, scale)
    decoded = zbardecode((qr_bytes, width, height))
    assert 1 == len(decoded)
    assert 'QRCODE' == decoded[0].type
    return decoded[0].data.decode('utf-8')


def test_issue_95():
    expected = 'WIFI:S:\\"foo\\;bar\\\\baz\\";;'
    data = helpers.make_wifi_data('"foo;bar\\baz"')
    assert data == expected
    if _qr_decoder_available:
        assert decode(data) == expected


# See <https://github.com/zxing/zxing/blob/master/core/src/test/java/com/google/zxing/client/result/WifiParsedResultTestCase.java#L38>  # noqa: E501
# See <https://github.com/zxing/zxing/blob/master/core/src/test/java/com/google/zxing/client/result/WifiParsedResultTestCase.java#L59>  # noqa: E501
@pytest.mark.parametrize('expected, ssid, password', (('WIFI:T:WEP;S:TenChars;P:0123456789;;',
                                                       'TenChars', '0123456789'),
                                                      ('WIFI:T:WEP;S:TenChars;P:abcde56789;;',
                                                       'TenChars', 'abcde56789'),
                                                      ('WIFI:T:WEP;S:TenChars;P:hellothere;;',
                                                       'TenChars', 'hellothere'),
                                                      ('WIFI:T:WEP;S:Ten\\;\\;Chars;P:0123456789;;',
                                                       'Ten;;Chars', '0123456789'),
                                                      ('WIFI:T:WEP;S:Ten\\:\\:Chars;P:0123456789;;',
                                                       'Ten::Chars', '0123456789'),
                                                      ('WIFI:T:WEP;S:TenChars;P:hellothere;;',
                                                       'TenChars', 'hellothere'),
                                                      ('WIFI:T:WEP;S:TenChars;P:hellothere;;',
                                                       'TenChars', 'hellothere'),
                                                      ('WIFI:T:WEP;S:Ten\\;\\;Chars;P:0123456789;;',
                                                       'Ten;;Chars', '0123456789'),
                                                      ('WIFI:T:WEP;S:Ten\\:\\:Chars;P:0123456789;;',
                                                       'Ten::Chars', '0123456789'),
                                                      # Escaped semicolons
                                                      ('WIFI:T:WEP;S:TenChars;P:hello\\;there;;',
                                                       'TenChars', 'hello;there'),
                                                      # Escaped colons
                                                      ('WIFI:T:WEP;S:TenChars;P:hello\\:there;;',
                                                       'TenChars', 'hello:there')
                                                      )
                         )
def test_wep(expected, ssid, password):
    data = helpers.make_wifi_data(ssid=ssid, password=password, security='WEP')
    assert data == expected
    if _qr_decoder_available:
        assert decode(data) == expected


# See <https://github.com/zxing/zxing/blob/master/core/src/test/java/com/google/zxing/client/result/WifiParsedResultTestCase.java#L56> # noqa: E501
@pytest.mark.parametrize('expected, ssid, password', (('WIFI:T:WPA;S:TenChars;P:wow;;',
                                                       'TenChars', 'wow'),
                                                      ('WIFI:T:WPA;S:TenChars;P:space is silent;;',
                                                       'TenChars', 'space is silent'),
                                                      )
                         )
def test_wpa(expected, ssid, password):
    data = helpers.make_wifi_data(ssid=ssid, password=password, security='WPA')
    assert data == expected
    if _qr_decoder_available:
        assert decode(data) == expected


# See <https://github.com/zxing/zxing/blob/master/core/src/test/java/com/google/zxing/client/result/WifiParsedResultTestCase.java#L68> # noqa: E501
@pytest.mark.parametrize('expected, ssid, password, security', (('WIFI:T:WPA;S:test;P:my_password\\\\;;',
                                                                 'test', 'my_password\\', 'WPA'),
                                                                ('WIFI:T:WPA;S:My_WiFi_SSID;P:abc123/;;',
                                                                 'My_WiFi_SSID', 'abc123/', 'WPA'),
                                                                ('WIFI:T:WPA;S:\\"foo\\;bar\\\\baz\\";;',
                                                                 '"foo;bar\\baz"', None, 'WPA')))
def test_escape(expected, ssid, password, security):
    data = helpers.make_wifi_data(ssid=ssid, password=password, security=security)
    assert data == expected
    if _qr_decoder_available:
        assert decode(data) == expected


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the encoder module.
"""
from __future__ import absolute_import, unicode_literals
import pytest
from segno import consts
from segno import encoder
from segno.encoder import Buffer
try:
    from .tutils import read_matrix
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from tutils import read_matrix


def bits(s):
    return bytearray([int(x, 2) for x in s if x != ' '])


def check_prepare_data(expected, data, mode, encoding):
    assert expected[1:] == tuple(encoder.prepare_data(data, mode, encoding))[1:]


def test_version_as_str():
    qr = encoder.encode('test', version='1', error=None, mode=None, mask=None,
                        eci=None, micro=None, encoding=None)
    assert 1 == qr.version


@pytest.mark.parametrize('data', ['1234567', '666'])
@pytest.mark.parametrize('mode', [None, consts.MODE_NUMERIC])
def test_prepare_data_numeric(data, mode):
    expected = ((data.encode('ascii'), len(data), consts.MODE_NUMERIC, None),)
    check_prepare_data(expected, data, mode, None)


@pytest.mark.parametrize('data', ['1234567', '666'])
@pytest.mark.parametrize('mode', [None, consts.MODE_NUMERIC,
                                  consts.MODE_ALPHANUMERIC, consts.MODE_BYTE])
def test_prepare_data_override_numeric(data, mode):
    encoding = None if mode != consts.MODE_BYTE else consts.DEFAULT_BYTE_ENCODING
    expected = ((data.encode('ascii'), len(data), mode or consts.MODE_NUMERIC, encoding),)
    check_prepare_data(expected, data, mode, None)


@pytest.mark.parametrize('data', (
        'HELLO WORLD',
        'ABCDEF',
        'HELLO    WORLD ',
        '-123',
    ))
@pytest.mark.parametrize('mode', [None, consts.MODE_ALPHANUMERIC])
def test_prepare_data_alphanumeric(data, mode):
    expected = ((data.encode('ascii'), len(data), consts.MODE_ALPHANUMERIC, None),)
    check_prepare_data(expected, data, mode, None)


@pytest.mark.parametrize('data', (
        'HELLO WORLD',
        'ABCDEF',
        'HELLO    WORLD ',
    ))
@pytest.mark.parametrize('mode', (None, consts.MODE_ALPHANUMERIC, consts.MODE_BYTE))
def test_prepare_data_override_alphanumeric(data, mode):
    encoding = None if mode != consts.MODE_BYTE else consts.DEFAULT_BYTE_ENCODING
    expected = ((data.encode('ascii'), len(data), mode or consts.MODE_ALPHANUMERIC, encoding),)
    check_prepare_data(expected, data, mode, None)


def _make_test_prepare_data_byte_data():
    test_data = (
        ('HeLLO WORLD', consts.DEFAULT_BYTE_ENCODING),
        ('HELLO\nWORLD', consts.DEFAULT_BYTE_ENCODING),
        ('☺', 'utf-8'),
        ('12345a', consts.DEFAULT_BYTE_ENCODING),
        ('abcdef', consts.DEFAULT_BYTE_ENCODING),
    )
    for data, encoding in test_data:
        for mode in (None, consts.MODE_BYTE):
            for param_encoding in (None, encoding):
                encoded_data = data.encode(encoding)
                expected = ((encoded_data, len(encoded_data), consts.MODE_BYTE, encoding),)
                yield expected, data, mode, param_encoding


@pytest.mark.parametrize('expected, data, mode, param_encoding',
                         _make_test_prepare_data_byte_data())
def test_prepare_data_byte(expected, data, mode, param_encoding):
    check_prepare_data(expected, data, mode, param_encoding)


def test_prepare_data_multiple():
    test_data = ['a', '1']
    segments = encoder.prepare_data(test_data, None, None)
    assert 2 == len(segments)
    assert 12 == segments.bit_length
    assert (1, consts.MODE_BYTE, consts.DEFAULT_BYTE_ENCODING) == segments[0][1:]
    assert (1, consts.MODE_NUMERIC, None) == segments[1][1:]


def test_prepare_data_multiple2():
    test_data = ['a']
    segments = encoder.prepare_data(test_data, None, None)
    assert 1 == len(segments)
    assert 8 == segments.bit_length
    assert (1, consts.MODE_BYTE, consts.DEFAULT_BYTE_ENCODING) == segments[0][1:]


def test_prepare_data_multiple_int():
    test_data = [1]
    segments = encoder.prepare_data(test_data, None, None)
    assert 1 == len(segments)
    assert 4 == segments.bit_length
    assert (1, consts.MODE_NUMERIC, None) == segments[0][1:]


def test_prepare_data_multiple_override():
    test_data = [(1, consts.MODE_ALPHANUMERIC), 2]
    segments = encoder.prepare_data(test_data, None, None)
    assert 2 == len(segments)
    assert 10 == segments.bit_length
    assert (1, consts.MODE_ALPHANUMERIC, None) == segments[0][1:]
    assert (1, consts.MODE_NUMERIC, None) == segments[1][1:]


def test_prepare_data_multiple_mode_none():
    test_data = [(1, None), ('A', None)]
    segments = encoder.prepare_data(test_data, None, None)
    assert 2 == len(segments)
    assert 10 == segments.bit_length
    assert (1, consts.MODE_NUMERIC, None) == segments[0][1:]
    assert (1, consts.MODE_ALPHANUMERIC, None) == segments[1][1:]


def test_prepare_data_multiple_mode_none_ignore_encoding():
    test_data = [(1, None, consts.DEFAULT_BYTE_ENCODING), ('A', None, 'utf-8')]
    segments = encoder.prepare_data(test_data, None, None)
    assert 2 == len(segments)
    assert 10 == segments.bit_length
    assert (1, consts.MODE_NUMERIC, None) == segments[0][1:]
    assert (1, consts.MODE_ALPHANUMERIC, None) == segments[1][1:]


def test_prepare_data_multiple_mode_none_encoding():
    test_data = [('Ä', None, consts.DEFAULT_BYTE_ENCODING), ('Ä', None, 'utf-8'),
                 ('Ä', None, None)]
    segments = encoder.prepare_data(test_data, None, None)
    assert 3 == len(segments)
    assert 32 == segments.bit_length
    seg1, seg2, seg3 = segments
    mode, encoding = consts.MODE_BYTE, consts.DEFAULT_BYTE_ENCODING
    assert (len('Ä'.encode(consts.DEFAULT_BYTE_ENCODING)), mode, encoding) == seg1[1:]
    encoding = 'utf-8'
    assert (len('Ä'.encode('utf-8')), mode, encoding) == seg2[1:]
    assert seg1 == seg3  # Encoding detection should produce the same result as 1st tuple


def test_prepare_data_multiple_tuple():
    test_data = ('a', '1')
    segments = encoder.prepare_data(test_data, None, None)
    assert 2 == len(segments)
    assert 12 == segments.bit_length
    assert (1, consts.MODE_BYTE, consts.DEFAULT_BYTE_ENCODING) == segments[0][1:]
    assert (1, consts.MODE_NUMERIC, None) == segments[1][1:]


def test_write_segment_eci_standard_value():
    # See ISO/IEC 18004:2006(E) -- 6.4.2.1 ECI Designator - EXAMPLE (page 24)
    encoding = 'ISO-8859-7'
    # TODO: ?
    # NOTE: ISO/IEC 18004:2006(E) uses "ΑΒΓΔΕ" but text says:
    # (character values A1HEX, A2HEX, A3HEX, A4HEX, A5HEX) and result seems
    # to use the A...HEX values as well
    s = '\u2018\u2019\u00A3\u20AC\u20AF'.encode(encoding)
    seg = encoder.make_segment(s, consts.MODE_BYTE, encoding)
    buff = Buffer()
    v, vrange = None, consts.VERSION_RANGE_01_09
    encoder.write_segment(buff, seg, v, vrange, True)
    expected = bits('0111 00001001 0100 00000101 10100001 10100010 10100011 10100100 10100101')
    result = buff.getbits()
    assert len(expected) == len(result)
    assert expected == result


@pytest.mark.parametrize('eci', [True, False])
def test_write_segment_numeric_standard_value_example1(eci):
    # See ISO/IEC 18004:2006(E) -- 6.4.3 Numeric mode - EXAMPLE 1 (page 25)
    s = b'01234567'
    seg = encoder.make_segment(s, consts.MODE_NUMERIC)
    buff = Buffer()
    v, vrange = None, consts.VERSION_RANGE_01_09
    encoder.write_segment(buff, seg, v, vrange, eci=eci)
    assert bits('00010000001000000000110001010110011000011') == buff.getbits()


@pytest.mark.parametrize('eci', [True, False])
def test_write_segment_numeric_standard_value_example2(eci):
    # See ISO/IEC 18004:2006(E) -- 6.4.3 Numeric mode - EXAMPLE 2 (page 25)
    s = b'0123456789012345'
    seg = encoder.make_segment(s, consts.MODE_NUMERIC)
    buff = Buffer()
    v, vrange = consts.VERSION_M3, consts.VERSION_M3
    encoder.write_segment(buff, seg, v, vrange, eci=eci)
    assert bits('0010000000000110001010110011010100110111000010100111010100101') == buff.getbits()


@pytest.mark.parametrize('eci', [True, False])
def test_write_segment_numeric_standard_value_i3(eci):
    # See ISO/IEC 18004:2006(E) -- I.3 Encoding a Micro QR Code symbol (page 96)
    s = b'01234567'
    seg = encoder.make_segment(s, consts.MODE_NUMERIC)
    buff = Buffer()
    v, vrange = consts.VERSION_M2, consts.VERSION_M2
    encoder.write_segment(buff, seg, v, vrange, eci=eci)
    assert bits('01000000000110001010110011000011') == buff.getbits()


@pytest.mark.parametrize('eci', [True, False])
def test_write_segment_alphanumeric_standard_value_example(eci):
    # See ISO/IEC 18004:2006(E) -- 6.4.3 Numeric mode - EXAMPLE (page 26)
    s = b'AC-42'
    seg = encoder.make_segment(s, consts.MODE_ALPHANUMERIC)
    buff = Buffer()
    v, vrange = None, consts.VERSION_RANGE_01_09
    encoder.write_segment(buff, seg, v, vrange, eci=eci)
    assert bits('00100000001010011100111011100111001000010') == buff.getbits()


@pytest.mark.parametrize('eci', [True, False])
def test_write_segment_alphanumeric_thonky(eci):
    # <http://www.thonky.com/qr-code-tutorial/data-encoding/#step-3-encode-using-the-selected-mode>
    s = b'HELLO WORLD'
    seg = encoder.make_segment(s, consts.MODE_ALPHANUMERIC, None)
    buff = Buffer()
    v, vrange = None, consts.VERSION_RANGE_01_09
    encoder.write_segment(buff, seg, v, vrange, eci)
    assert bits('00100000010110110000101101111000110100010111001011011100010011010100001101') == buff.getbits()


@pytest.mark.parametrize('eci', [True, False])
def test_write_segment_bytes_thonky(eci):
    # <http://www.thonky.com/qr-code-tutorial/byte-mode-encoding/>
    s = b'Hello, world!'
    seg = encoder.make_segment(s, consts.MODE_BYTE, consts.DEFAULT_BYTE_ENCODING)
    buff = Buffer()
    v, vrange = None, consts.VERSION_RANGE_01_09
    encoder.write_segment(buff, seg, v, vrange, eci=eci)
    assert bits('010000001101'
                '01001000'
                '01100101'
                '01101100'
                '01101100'
                '01101111'
                '00101100'
                '00100000'
                '01110111'
                '01101111'
                '01110010'
                '01101100'
                '01100100'
                '00100001') == buff.getbits()


def test_write_terminator_thonky():
    # <http://www.thonky.com/qr-code-tutorial/data-encoding/#add-a-terminator-of-0s-if-necessary>
    data = bits('00100000010110110000101101111000110100010111001011011100010011010100001101')
    buff = Buffer(data)
    version = 1
    v = None
    capacity = consts.SYMBOL_CAPACITY[version][consts.ERROR_LEVEL_Q]
    encoder.write_terminator(buff, capacity, v, len(data))
    assert data + bits('0000') == buff.getbits()


def test_write_terminator_standard_value_i2():
    # See ISO/IEC 18004:2006(E) -- I.2 Encoding a QR Code symbol (page 94)
    data = bits('000100000010000000001100010101100110000110000')
    buff = Buffer(data)
    version = 1
    v = None
    capacity = consts.SYMBOL_CAPACITY[version][consts.ERROR_LEVEL_M]
    encoder.write_terminator(buff, capacity, v, len(data))
    assert data + bits('0000') == buff.getbits()


def test_write_terminator_standard_value_i3():
    # See ISO/IEC 18004:2006(E) -- I.3 Encoding a Micro QR Code symbol (page 96)
    data = bits('01000000000110001010110011000011')
    buff = Buffer(data)
    version = consts.VERSION_M2
    capacity = consts.SYMBOL_CAPACITY[version][consts.ERROR_LEVEL_L]
    encoder.write_terminator(buff, capacity, version, len(data))
    assert data + bits('00000') == buff.getbits()


def test_write_padding_bits_iso_i2():
    # See ISO/IEC 18004:2006(E) -- I.2 Encoding a QR Code symbol (page 94)
    data = bits('0001 0000001000 0000001100 0101011001 1000011 0000')
    buff = Buffer(data)
    version = 1
    encoder.write_padding_bits(buff, version, len(buff))
    assert bits('00010000 00100000 00001100 01010110 01100001 10000000') == buff.getbits()


def test_write_padding_bits_iso_i3():
    # See ISO/IEC 18004:2006(E) -- I.3 Encoding a Micro QR Code symbol (page 96)
    data = bits('0 1000 0000001100 0101011001 1000011 00000')
    buff = Buffer(data)
    version = consts.VERSION_M2
    encoder.write_padding_bits(buff, version, len(buff))
    assert bits('01000000 00011000 10101100 11000011 00000000') == buff.getbits()


def test_write_padding_bits_thonky():
    # <http://www.thonky.com/qr-code-tutorial/data-encoding>
    data = bits('00100000 01011011 00001011 01111000 11010001 01110010 11011100 '
                '01001101 01000011 010000')
    buff = Buffer(data)
    version = 1
    encoder.write_padding_bits(buff, version, len(buff))
    assert bits('00100000 01011011 00001011 01111000 11010001 01110010 11011100 '
                '01001101 01000011 01000000') == buff.getbits()


def test_write_pad_codewords_standard_value_i2():
    # See ISO/IEC 18004:2006(E) -- I.2 Encoding a QR Code symbol (page 94)
    data = bits('00010000 00100000 00001100 01010110 01100001 10000000')
    buff = Buffer(data)
    version = 1
    error = consts.ERROR_LEVEL_M
    capacity = consts.SYMBOL_CAPACITY[version][error]
    encoder.write_pad_codewords(buff, version, capacity, len(buff))
    assert data + bits('1110110000010001111011000001000111101100000100011110110'
                       '0000100011110110000010001') == buff.getbits()


def test_write_pad_codewords_standard_value_i3():
    # See ISO/IEC 18004:2006(E) -- I.3 Encoding a Micro QR Code symbol (page 96)
    data = bits('0100000000011000101011001100001100000000')
    buff = Buffer(data)
    version = consts.VERSION_M2
    error = consts.ERROR_LEVEL_L
    capacity = consts.SYMBOL_CAPACITY[version][error]
    encoder.write_pad_codewords(buff, version, capacity, len(buff))
    assert data == buff.getbits()


@pytest.mark.parametrize('data', [b'ABCDEF', b'A', b'A1', b'1A', b'HTTP://WWW.EXAMPLE.ORG',
                                  b'666', b' ', b'HELLO WORLD', b'AC-42',
                                  consts.ALPHANUMERIC_CHARS])
def test_is_alphanumeric(data):
    assert encoder.is_alphanumeric(data)


@pytest.mark.parametrize('data', [b'a', b'0123b', b'_', b'http://www.example.org/',  b'',
                                  'ü'.encode('utf-8'), 'Ü'.encode('utf-8'),
                                  'ä'.encode(consts.DEFAULT_BYTE_ENCODING), b'HELLO\nWORLD', b'Ab'])
def test_is_not_alphanumeric(data):
    assert not encoder.is_alphanumeric(data)


@pytest.mark.parametrize('data', ['点', '漢字', '外来語', 'あめかんむり', 'ひへん', 'くさかんむり'])
def test_is_kanji(data):
    assert encoder.is_kanji(data.encode('shift_jis'))


@pytest.mark.parametrize('data', ['abcdef', '1234', '1234a', 'ABCDEF'])
def test_is_not_kanji(data):
    assert not encoder.is_kanji(data.encode('shift_jis'))


_test_find_mode_test_data = (
        (b'123', consts.MODE_NUMERIC),
        (b'-123', consts.MODE_ALPHANUMERIC),
        (b'123a', consts.MODE_BYTE),
        (b'123A', consts.MODE_ALPHANUMERIC),
        (b'HELLO WORLD', consts.MODE_ALPHANUMERIC),
        (b'HELLO \n WORLD', consts.MODE_BYTE),
        (b'HTTP://WWW.EXAMPLE.ORG/', consts.MODE_ALPHANUMERIC),
        (b'http://www.example.org/', consts.MODE_BYTE),
        (b'#', consts.MODE_BYTE),
        (b'', consts.MODE_BYTE),
        (b'a', consts.MODE_BYTE),
        ('点'.encode('shift_jis'), consts.MODE_KANJI),
        ('茗'.encode('shift_jis'), consts.MODE_KANJI),
        ('漢字'.encode('shift_jis'), consts.MODE_KANJI),
        ('外来語'.encode('shift_jis'), consts.MODE_KANJI),
        ('外来語'.encode('utf-8'), consts.MODE_BYTE),
    )


@pytest.mark.parametrize('data, expected', _test_find_mode_test_data)
def test_find_mode(data, expected):
    assert expected == encoder.find_mode(data)


@pytest.mark.parametrize('version, expected',  # Version, matrix size
                                               [('M1', 11),
                                                ('M2', 13),
                                                ('M3', 15),
                                                ('M4', 17),
                                                (1, 21),
                                                (27, 125),
                                                (40, 177)])
def test_get_matrix_size(version, expected):
    try:
        v = consts.MICRO_VERSION_MAPPING[version]
    except KeyError:
        v = version
    assert expected == encoder.calc_matrix_size(v)


# See ISO/IEC 18004:2006(E) - Table 2 (page 22)
_test_is_mode_supported_micro_data = (
    (consts.MODE_NUMERIC, 'M1', True),
    (consts.MODE_NUMERIC, 'M2', True),
    (consts.MODE_NUMERIC, 'M3', True),
    (consts.MODE_NUMERIC, 'M4', True),
    (consts.MODE_ALPHANUMERIC, 'M1', False),
    (consts.MODE_ALPHANUMERIC, 'M2', True),
    (consts.MODE_ALPHANUMERIC, 'M3', True),
    (consts.MODE_ALPHANUMERIC, 'M4', True),
    (consts.MODE_ECI, 'M1', False),
    (consts.MODE_ECI, 'M2', False),
    (consts.MODE_ECI, 'M3', False),
    (consts.MODE_ECI, 'M4', False),
    (consts.MODE_BYTE, 'M1', False),
    (consts.MODE_BYTE, 'M2', False),
    (consts.MODE_BYTE, 'M3', True),
    (consts.MODE_BYTE, 'M4', True),
    (consts.MODE_KANJI, 'M3', True),
    (consts.MODE_KANJI, 'M4', True),
)


@pytest.mark.parametrize('mode, version, expected', _test_is_mode_supported_micro_data)
def test_is_mode_supported_micro(mode, version, expected):
    v = consts.MICRO_VERSION_MAPPING[version]
    assert expected == encoder.is_mode_supported(mode, v)


@pytest.mark.parametrize('version', tuple(range(1, 41)))
@pytest.mark.parametrize('mode', (consts.MODE_NUMERIC, consts.MODE_ALPHANUMERIC,
                                  consts.MODE_BYTE, consts.MODE_ECI, consts.MODE_KANJI))
def test_is_mode_supported_micro2(version, mode):
    assert encoder.is_mode_supported(mode, version)


@pytest.mark.parametrize('mode, version', [(-1, 1), (-2, consts.VERSION_M2),
                                           (10, 1), (10, consts.VERSION_M1),
                                           (9, 39)])
def test_is_mode_supported_invalid_mode(mode, version):
    with pytest.raises(ValueError) as ex:
        encoder.is_mode_supported(mode, version)
    assert 'mode' in str(ex.value)


@pytest.mark.parametrize('mode', ('kanij', 'binary', 'blub', ''))
def test_normalize_mode_illegal(mode):
    with pytest.raises(ValueError) as ex:
        encoder.normalize_mode(mode)
    assert 'mode' in str(ex.value)


@pytest.mark.parametrize('mask', tuple(range(8)))
def test_normalize_mask(mask):
    assert mask == encoder.normalize_mask(mask, is_micro=False)


@pytest.mark.parametrize('mask', tuple(range(4)))
def test_normalize_mask_micro(mask):
    assert mask == encoder.normalize_mask(mask, is_micro=True)


def test_normalize_mask_none():
    assert encoder.normalize_mask(None, is_micro=True) is None
    assert encoder.normalize_mask(None, is_micro=False) is None


@pytest.mark.parametrize('version, mask', [(consts.VERSION_M1, 8), (1, 9), (1, -1)])
def test_normalize_mask_illegal(version, mask):
    with pytest.raises(ValueError):
        encoder.normalize_mask(mask, version < 1)


def test_mode_name_illegal():
    with pytest.raises(ValueError):
        encoder.get_mode_name(7)


def test_error_name_illegal():
    with pytest.raises(ValueError):
        encoder.get_error_name(7)


def test_version_name_illegal():
    with pytest.raises(ValueError):
        encoder.get_version_name(41)


def test_version_range_illegal():
    with pytest.raises(ValueError):
        encoder.version_range(41)


def test_normalize_errorlevel():
    assert consts.ERROR_LEVEL_H == encoder.normalize_errorlevel('h')
    assert consts.ERROR_LEVEL_Q == encoder.normalize_errorlevel('Q')
    assert encoder.normalize_errorlevel(None, accept_none=True) is None
    assert consts.ERROR_LEVEL_M == encoder.normalize_errorlevel(consts.ERROR_LEVEL_M)


def test_normalize_errorlevel_illegal():
    with pytest.raises(ValueError):
        encoder.normalize_errorlevel('g')


def test_normalize_errorlevel_illegal2():
    with pytest.raises(ValueError) as ex:
        encoder.normalize_errorlevel(None)
    assert 'error' in str(ex.value)


_test_find_version_test_data = (
    # data, error, micro, expected version
    ('12345', None, True, consts.VERSION_M1),
    ('12345', consts.ERROR_LEVEL_L, True, consts.VERSION_M2),
    # Error level Q isn't suppoted by M1 - M3
    ('12345', consts.ERROR_LEVEL_Q, True, consts.VERSION_M4),
    # Error level H isn't supported by Micro QR Codes
    ('12345', consts.ERROR_LEVEL_H, None, 1),
    ('12345', None, False, 1),
    (12345, None, True, consts.VERSION_M1),
    (-12345, None, True, consts.VERSION_M2),  # Negative number
    (-12345, consts.ERROR_LEVEL_M, True, consts.VERSION_M3),  # Negative number
    (12345, None, False, 1),
    ('123456', None, True, consts.VERSION_M2),
    ('123456', None, False, 1),
    (123456, None, True, consts.VERSION_M2),
    (123456, None, False, 1),
    ('ABCDE', None, True, consts.VERSION_M2),  # Alphanumeric isn't supported by M1
    ('ABCDEF', consts.ERROR_LEVEL_L, True, consts.VERSION_M2),
    ('ABCDEF', consts.ERROR_LEVEL_M, True, consts.VERSION_M3),  # Too much data for error level M and version M2
    ('ABCDEF', consts.ERROR_LEVEL_L, False, 1),
    ('ABCDEF', consts.ERROR_LEVEL_L, False, 1),
    ('Märchenbuch', None, True, consts.VERSION_M4),
    ('Märchenbücher', None, False, 1),
    ('Märchenbücherei', None, None, consts.VERSION_M4),
    ('Märchenbücherei', consts.ERROR_LEVEL_M, None, 2),
)


@pytest.mark.parametrize('data, error, micro, expected_version', _test_find_version_test_data)
def test_find_version(data, error, micro, expected_version):
    segments = encoder.prepare_data(data, None, None)
    assert expected_version == encoder.find_version(segments, error, eci=False, micro=micro)


def test_thonky_add_format_info():
    # <http://www.thonky.com/qr-code-tutorial/format-version-information#put-the-format-string-into-the-qr-code>
    version = 1
    matrix = encoder.make_matrix(version, reserve_regions=False)
    encoder.add_finder_patterns(matrix, is_micro=False)
    encoder.add_format_info(matrix, version, consts.ERROR_LEVEL_L, 4)
    ref_matrix = read_matrix('thonky_format')
    assert ref_matrix == matrix


def test_thonky_add_version_info():
    # <http://www.thonky.com/qr-code-tutorial/format-version-information>
    version = 7
    matrix = encoder.make_matrix(version, reserve_regions=False)
    encoder.add_finder_patterns(matrix, is_micro=False)
    encoder.add_alignment_patterns(matrix, version)
    encoder.add_version_info(matrix, version)
    matrix[-8][8] = 0x1  # dark module
    ref_matrix = read_matrix('thonky_version')
    assert ref_matrix == matrix


def test_eval_micro():
    # ISO/IEC 18004:2006(E) page 54
    # 6.8.2.2 Evaluation of Micro QR Code symbols
    matrix = read_matrix('iso_6.8.2.2')
    res = encoder.evaluate_micro_mask(matrix, len(matrix))
    assert 104 == res


def make_thonky_score_matrix():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking-1')
    return matrix, len(matrix)


def test_score_n1():
    matrix, matrix_size = make_thonky_score_matrix()
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[0]
    assert 180 == score


def test_score_n2():
    matrix, matrix_size = make_thonky_score_matrix()
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[1]
    assert 90 == score


def test_score_n3():
    matrix, matrix_size = make_thonky_score_matrix()
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[2]
    # Thonky: 80
    assert 760 == score


def test_score_n4():
    matrix = read_matrix('thonky_datamasking-2')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[3]
    assert 0 == score


def test_thonky_pattern_0():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-0')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 180 == scores[0]
    assert 90 == scores[1]
    # Thonky: 80
    assert 760 == scores[2]
    assert 0 == scores[3]
    # See score 3
    assert 350 - 80 + 760 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_1():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-1')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 172 == scores[0]
    assert 129 == scores[1]
    # Thonky: 120
    assert 760 == scores[2]
    assert 0 == scores[3]
    # See score 3
    assert 421 - 120 + 760 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_2():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-2')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 206 == scores[0]
    assert 141 == scores[1]
    # Thonky: 160
    assert 800 == scores[2]
    assert 0 == scores[3]
    assert 507 - 160 + 800 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_3():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-3')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 180 == scores[0]
    assert 141 == scores[1]
    # Thonky: 120
    assert 760 == scores[2]
    # Thonky: 2, but that's impossible: Either 0 or a multiple of 10 (N4 = 10)
    assert 0 == scores[3]
    assert 443 - 2 - 120 + 760 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_4():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-4')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 195 == scores[0]
    assert 138 == scores[1]
    # Thonky: 200
    assert 800 == scores[2]
    assert 0 == scores[3]
    # See score 3
    assert 533 - 200 + 800 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_5():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-5')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 189 == scores[0]
    assert 156 == scores[1]
    # Thonky: 200
    assert 800 == scores[2]
    # Thonky: 2, but that's impossible: Either 0 or a multiple of 10 (N4 = 10)
    assert 0 == scores[3]
    # See score 3 and 4
    assert 547 - 2 - 200 + 800 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_6():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-6')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 171 == scores[0]
    assert 102 == scores[1]
    # Thonky: 80
    assert 840 == scores[2]
    # Thonky: 4, but that's impossible: Either 0 or a multiple of 10 (N4 = 10)
    assert 0 == scores[3]
    # See score 3 and 4
    assert 357 - 4 - 80 + 840 == encoder.evaluate_mask(matrix, matrix_size)


def test_thonky_pattern_7():
    # http://www.thonky.com/qr-code-tutorial/data-masking
    matrix = read_matrix('thonky_datamasking_mask-7')
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    assert 197 == scores[0]
    assert 123 == scores[1]
    # Thonky: 200
    assert 720 == scores[2]
    assert 0 == scores[3]
    # See score 3
    assert 520 - 200 + 720 == encoder.evaluate_mask(matrix, matrix_size)


def test_score_n1_iso():
    # For example, impose 5 penalty points on the block of “dark:dark:dark:dark:dark:dark:dark”
    # module pattern, where a series of seven consecutive modules is counted as one block
    matrix = tuple(bytearray([1] * 7) for i in range(7))
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[0]
    assert 5 * 7 * 2 == score


def test_score_n2_iso():
    # Take a block consisting of 3 x 3 dark
    # modules for an example. Considering that up to four 2 x 2 dark modules can
    # be included in this block, the penalty applied to this block shall be
    # calculated as 4 (blocks) x 3 (points) = 12 points.
    matrix = (bytearray([1, 1, 1]), bytearray([1, 1, 1]), bytearray([1, 1, 1]))
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[1]
    assert 12 == score

    matrix = (bytearray([0, 0, 0]), bytearray([0, 0, 0]), bytearray([0, 0, 0]))
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[1]
    assert 12 == score


def _make_score_n4_data():
    for p in range(41, 46):
        yield 10, p

    for p in range(55, 60):
        yield 10, p

    for p in range(46, 55):
        yield 0, p

    yield 3 * 10, 35


@pytest.mark.parametrize('score, percent', _make_score_n4_data())
def test_score_n4_iso(score, percent):
    # Add 10 points to a deviation of 5% increment or decrement in the
    # proportion ratio of dark module from the referential 50% (or 0 point)
    # level. For example, assign 0 points as a penalty if the ratio of dark
    # module is between 45% and 55%, or 10 points if the ratio of dark module
    # is between 40% and 60%

    def make_matrix():
        row = [0x0] * 10
        return tuple([bytearray(row) for i in range(10)])

    def fill_matrix(matrix, percent):
        cnt = 0
        finished = False
        for i in range(len(matrix)):
            for j in range(len(matrix)):
                matrix[i][j] = 0x1
                cnt += 1
                finished = cnt == percent
                if finished:
                    break
            if finished:
                break

    matrix = make_matrix()
    fill_matrix(matrix, percent)
    matrix_size = len(matrix)
    scores = encoder.mask_scores(matrix, matrix_size)
    score = scores[3]
    assert score == score


def test_binary_sequence_to_integers():
    # <http://www.thonky.com/qr-code-tutorial/error-correction-coding>
    # HELLO WORLD as a 1-M code
    data = bits('00100000 01011011 00001011 01111000 11010001 01110010 11011100 '
                '01001101 01000011 01000000 11101100 00010001 11101100 00010001 '
                '11101100 00010001')
    expected = [32, 91, 11, 120, 209, 114, 220, 77, 67, 64, 236, 17, 236, 17, 236, 17]
    res_int = list(Buffer(data).toints())
    assert len(data) // 8 == len(res_int)
    assert expected == res_int


def test_split_into_blocks():
    # <http://www.thonky.com/qr-code-tutorial/error-correction-coding>
    # HELLO WORLD as a 5-Q code
    s = '01000011 01010101 01000110 10000110 01010111 00100110 01010101 ' \
        '11000010 01110111 00110010 00000110 00010010 00000110 01100111 ' \
        '00100110 11110110 11110110 01000010 00000111 01110110 10000110 ' \
        '11110010 00000111 00100110 01010110 00010110 11000110 11000111 ' \
        '10010010 00000110 10110110 11100110 11110111 01110111 00110010 ' \
        '00000111 01110110 10000110 01010111 00100110 01010010 00000110 ' \
        '10000110 10010111 00110010 00000111 01000110 11110111 01110110 ' \
        '01010110 11000010 00000110 10010111 00110010 11100000 11101100 ' \
        '00010001 11101100 00010001 11101100 00010001 11101100'
    data = bits(s)
    buff = Buffer(data)
    codewords = list(buff.toints())
    assert 62 == len(codewords)
    ec_infos = consts.ECC[5][consts.ERROR_LEVEL_Q]
    assert ec_infos
    assert 2 == len(ec_infos)
    blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert 4 == len(blocks)
    assert 15 == len(blocks[0])
    assert bytearray(codewords[:15]) == blocks[0]
    assert 15 == len(blocks[1])
    assert 16 == len(blocks[2])
    assert 16 == len(blocks[3])


def test_make_error_block0():
    # <http://www.thonky.com/qr-code-tutorial/error-correction-coding>
    # 1-M
    codeword = '00100000 01011011 00001011 01111000 11010001 01110010 11011100 ' \
               '01001101 01000011 01000000 11101100 00010001 11101100 00010001 ' \
               '11101100 00010001'
    codeword_ints = [32, 91, 11, 120, 209, 114, 220, 77, 67, 64, 236, 17, 236, 17, 236, 17]
    buff = Buffer(bits(codeword))
    assert codeword_ints == list(buff.toints())
    error_block = bytearray([196, 35, 39, 119, 235, 215, 231, 226, 93, 23])
    ec_infos = consts.ECC[1][consts.ERROR_LEVEL_M]
    assert 1 == len(ec_infos)
    ec_info = ec_infos[0]
    assert 1 == ec_info.num_blocks
    assert 10 == ec_info.num_total - ec_info.num_data
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert len(error_block) == len(error_blocks[0])
    assert error_block == error_blocks[0]


def test_make_error_block1():
    # <http://www.thonky.com/qr-code-tutorial/structure-final-message>
    # 5-Q
    codeword = '01000011 01010101 01000110 10000110 01010111 00100110 01010101 ' \
               '11000010 01110111 00110010 00000110 00010010 00000110 01100111 ' \
               '00100110'
    codeword_ints = [67, 85, 70, 134, 87, 38, 85, 194, 119, 50, 6, 18, 6, 103, 38]
    buff = Buffer(bits(codeword))
    assert codeword_ints == list(buff.toints())
    error_block = bytearray([213, 199, 11, 45, 115, 247, 241, 223, 229, 248, 154, 117, 154, 111, 86, 161, 111, 39])
    ec_infos = consts.ECC[5][consts.ERROR_LEVEL_Q]
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert error_block == error_blocks[0]


def test_make_error_block2():
    # <http://www.thonky.com/qr-code-tutorial/structure-final-message>
    # 5-Q
    codeword = '11110110 11110110 01000010 00000111 01110110 10000110 11110010 ' \
               '00000111 00100110 01010110 00010110 11000110 11000111 10010010 ' \
               '00000110'
    codeword_ints = [246, 246, 66, 7, 118, 134, 242, 7, 38, 86, 22, 198, 199,
                     146, 6]
    buff = Buffer(bits(codeword))
    assert codeword_ints == list(buff.toints())
    error_block = bytearray([87, 204, 96, 60, 202, 182, 124, 157, 200, 134, 27,
                             129, 209, 17, 163, 163, 120, 133])
    ec_infos = consts.ECC[5][consts.ERROR_LEVEL_Q]
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert error_block == error_blocks[0]


def test_make_error_block3():
    # <http://www.thonky.com/qr-code-tutorial/structure-final-message>
    # 5-Q
    codeword = '10110110 11100110 11110111 01110111 00110010 00000111 01110110 ' \
               '10000110 01010111 00100110 01010010 00000110 10000110 10010111 ' \
               '00110010 00000111'
    codeword_ints = [182, 230, 247, 119, 50, 7, 118, 134, 87, 38, 82, 6, 134,
                     151, 50, 7]
    buff = Buffer(bits(codeword))
    assert codeword_ints == list(buff.toints())
    error_block = bytearray([148, 116, 177, 212, 76, 133, 75, 242, 238, 76,
                             195, 230, 189, 10, 108, 240, 192, 141])
    ec_infos = (consts.ECC[5][consts.ERROR_LEVEL_Q][1],)
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert error_block == error_blocks[0]


def test_make_error_block4():
    # <http://www.thonky.com/qr-code-tutorial/structure-final-message>
    # 5-Q
    codeword = '01000110 11110111 01110110 01010110 11000010 00000110 10010111 ' \
               '00110010 00010000 11101100 00010001 11101100 00010001 11101100 ' \
               '00010001 11101100'
    codeword_ints = [70, 247, 118, 86, 194, 6, 151, 50, 16, 236, 17, 236, 17,
                     236, 17, 236]
    buff = Buffer(bits(codeword))
    assert codeword_ints == list(buff.toints())
    error_block = bytearray([235, 159, 5, 173, 24, 147, 59, 33, 106, 40, 255,
                             172, 82, 2, 131, 32, 178, 236])
    ec_infos = (consts.ECC[5][consts.ERROR_LEVEL_Q][1],)
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert error_block == error_blocks[0]


def test_make_error_block_iso_i2():
    # ISO/IEC 18004:2006(E) - I.2 Encoding a QR Code symbol -- page 94
    # Input: 01234567
    # Symbol: 1-M
    s = '00010000 00100000 00001100 01010110 01100001 10000000 11101100 ' \
        '00010001 11101100 00010001 11101100 00010001 11101100 00010001 ' \
        '11101100 00010001'
    data_block = Buffer(bits(s))
    error_s = '10100101 00100100 11010100 11000001 11101101 00110110 11000111 ' \
              '10000111 00101100 01010101'
    error_block = list(Buffer(bits(error_s)).toints())
    ec_infos = consts.ECC[1][consts.ERROR_LEVEL_M]
    ec_info = ec_infos[0]
    assert ec_info.num_total - ec_info.num_data == len(error_block)
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, data_block)
    assert bytearray(error_block) == error_blocks[0]


def test_make_error_block_iso_i3():
    # ISO/IEC 18004:2006(E) - I.3 Encoding a Micro QR Code symbol  -- page 96
    # Input: 01234567
    # Symbol: M2-L
    s = '01000000 00011000 10101100 11000011 00000000'
    buff = Buffer(bits(s))
    error_s = '10000110 00001101 00100010 10101110 00110000'
    error_block = list(Buffer(bits(error_s)).toints())
    ec_infos = consts.ECC[consts.VERSION_M2][consts.ERROR_LEVEL_L]
    ec_info = ec_infos[0]
    assert ec_info.num_total - ec_info.num_data == len(error_block)
    data_blocks, error_blocks = encoder.make_blocks(ec_infos, buff)
    assert bytearray(error_block) == error_blocks[0]


def test_make_final_message_iso_i2():
    # ISO/IEC 18004:2015(E) - I.2 Encoding a QR Code symbol  -- page 94
    # Input: 01234567
    # Symbol: 1-M
    s = '00010000 00100000 00001100 01010110 01100001 10000000 11101100 ' \
        '00010001 11101100 00010001 11101100 00010001 11101100 00010001 ' \
        '11101100 00010001'
    codewords = Buffer(bits(s))
    expected_s = '00010000 00100000 00001100 01010110 01100001 10000000 ' \
                 '11101100 00010001 11101100 00010001 11101100 00010001 ' \
                 '11101100 00010001 11101100 00010001 10100101 00100100 ' \
                 '11010100 11000001 11101101 00110110 11000111 10000111 ' \
                 '00101100 01010101'
    expected = bits(expected_s)
    assert expected == encoder.make_final_message(1, consts.ERROR_LEVEL_M,
                                                  codewords).getbits()


def test_make_final_message_iso_i3():
    # ISO/IEC 18004:2015(E) - I.3 Encoding a Micro QR Code symbol  -- page 96
    # Input: 01234567
    # Symbol: M2-L
    s = '01000000 00011000 10101100 11000011 00000000'
    codewords = Buffer(bits(s))
    expected_s = '01000000 00011000 10101100 11000011 00000000 10000110 ' \
                 '00001101 00100010 10101110 00110000'
    expected = bits(expected_s)
    assert expected == encoder.make_final_message(consts.VERSION_M2,
                                                  consts.ERROR_LEVEL_L,
                                                  codewords).getbits()


def test_make_final_message_thonky():
    # <http://www.thonky.com/qr-code-tutorial/structure-final-message>
    # 5-Q
    codewords = '01000011 01010101 01000110 10000110 01010111 00100110 01010101 ' \
                '11000010 01110111 00110010 00000110 00010010 00000110 01100111 ' \
                '00100110 11110110 11110110 01000010 00000111 01110110 10000110 ' \
                '11110010 00000111 00100110 01010110 00010110 11000110 11000111 ' \
                '10010010 00000110 10110110 11100110 11110111 01110111 00110010 ' \
                '00000111 01110110 10000110 01010111 00100110 01010010 00000110 ' \
                '10000110 10010111 00110010 00000111 01000110 11110111 01110110 ' \
                '01010110 11000010 00000110 10010111 00110010 00010000 11101100 ' \
                '00010001 11101100 00010001 11101100 00010001 11101100'
    codewords_int = [67, 85, 70, 134, 87, 38, 85, 194, 119, 50, 6, 18, 6, 103, 38,
                     246, 246, 66, 7, 118, 134, 242, 7, 38, 86, 22, 198, 199, 146, 6,
                     182, 230, 247, 119, 50, 7, 118, 134, 87, 38, 82, 6, 134, 151, 50, 7,
                     70, 247, 118, 86, 194, 6, 151, 50, 16, 236, 17, 236, 17, 236, 17, 236]
    s = '0100001111110110101101100100011001010101111101101110011011110111010001' \
        '1001000010111101110111011010000110000001110111011101010110010101110111' \
        '0110001100101100001000100110100001100000011100000110010101011111001001' \
        '1101101001011111000010000001111000011000110010011101110010011001010111' \
        '0001000000110010010101100010011011101100000001100001011001010010000100' \
        '0100010010110001100000011011101100000001101100011110000110000100010110' \
        '0111100100101001011111101100001001100000011000110010000100010000011111' \
        '1011001101010101010111100101001110101111000111110011000111010010011111' \
        '0000101101100000101100010000010100101101001111001101010010101101011100' \
        '1111001010010011000001100011110111101101101000010110010011111100010111' \
        '1100010010110011101111011111100111011111001000100001111001011100100011' \
        '1011100110101011111000100001100100110000101000100110100001101111000011' \
        '1111111101110101100000011110011010101100100110101101000110111101010100' \
        '1001101111000100010000101000000010010101101010001101101100100000111010' \
        '0001101000111111000000100000011011110111100011000000101100100010011110' \
        '00010110001101111011000000000'
    expected = bits(s)
    buff = Buffer(bits(codewords))
    assert codewords_int == list(buff.toints())
    res = encoder.make_final_message(5, consts.ERROR_LEVEL_Q, buff)
    assert len(expected) == len(res)
    assert expected == res.getbits()


def test_encode_iso_fig1():
    # ISO/IEC 18004:2015(E) - page 7
    # 'QR Code Symbol' as 1-M symbol
    qr = encoder.encode('QR Code Symbol', error='M', mask=None, micro=False,
                        boost_error=False)
    assert consts.ERROR_LEVEL_M == qr.error
    assert 1 == qr.version
    assert 5 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
    ref_matrix = read_matrix('iso-fig-1')
    assert ref_matrix == qr.matrix


def test_encode_iso_i2():
    # ISO/IEC 18004:2015(E) - page 94
    # 01234567 as 1-M symbol
    # TODO: Without the mask param Segno chooses mask 3 which seems to be correct
    # Mask 2 is IMO an error in the standard
    qr = encoder.encode('01234567', error='m', version=1, mask=2, micro=False,
                        boost_error=False)
    assert consts.ERROR_LEVEL_M == qr.error
    assert 1 == qr.version
    assert 2 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
    qr = encoder.encode('01234567', error='m', mask=2, micro=False,
                        boost_error=False)
    assert consts.ERROR_LEVEL_M == qr.error
    assert 1 == qr.version
    assert 2 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
    ref_matrix = read_matrix('iso-i2')
    assert ref_matrix == qr.matrix


def test_encode_iso_i3():
    # ISO/IEC 18004:2015(E) - page 96
    # 01234567 as M2-L symbol
    ref_matrix = read_matrix('iso-i3')
    qr = encoder.encode('01234567', error='l', version='m2', boost_error=False)
    assert consts.ERROR_LEVEL_L == qr.error
    assert consts.VERSION_M2 == qr.version
    assert 1 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
    assert ref_matrix == qr.matrix
    qr = encoder.encode('01234567', error='l', version=None, micro=True,
                        boost_error=False)
    assert consts.ERROR_LEVEL_L == qr.error
    assert consts.VERSION_M2 == qr.version
    assert 1 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
    assert ref_matrix == qr.matrix


def test_encode_iso_fig29():
    # ISO/IEC 18004:2015(E) - page 60
    # ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ

    # TODO: If mask is None, Segno chooses mask 3, but the figure uses mask 4...
    qr = encoder.encode('ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                        error='m', mask=4, boost_error=False)
    assert 4 == qr.mask
    assert 4 == qr.version
    ref_matrix = read_matrix('iso-fig-29')
    assert ref_matrix == qr.matrix


def test_codeword_placement_iso_i2():
    # ISO/IEC 18004:2015(E) - page 94
    # 01234567 as 1-M symbol
    s = '00010000 00100000 00001100 01010110 01100001 10000000 11101100 00010001 ' \
        '11101100 00010001 11101100 00010001 11101100 00010001 11101100 00010001'
    codewords = Buffer(bits(s))
    version = 1
    buff = encoder.make_final_message(version, consts.ERROR_LEVEL_M, codewords)
    expected_s = '00010000 00100000 00001100 01010110 01100001 10000000 11101100 ' \
                 '00010001 11101100 00010001 11101100 00010001 11101100 00010001 ' \
                 '11101100 00010001 10100101 00100100 11010100 11000001 11101101 ' \
                 '00110110 11000111 10000111 00101100 01010101'
    expected = bits(expected_s)
    assert expected == buff.getbits()
    matrix = encoder.make_matrix(version)
    encoder.add_finder_patterns(matrix, is_micro=version < 1)
    encoder.add_codewords(matrix, buff, version=version)
    ref_matrix = read_matrix('iso-i2_code_placement')
    assert ref_matrix == matrix


def test_codeword_placement_iso_i3():
    # ISO/IEC 18004:2015(E) - page 96
    # 01234567 as M2-L symbol
    s = '01000000 00011000 10101100 11000011 00000000'
    codewords = Buffer(bits(s))
    version = consts.VERSION_M2
    buff = encoder.make_final_message(version, consts.ERROR_LEVEL_L, codewords)
    expected_s = '01000000 00011000 10101100 11000011 00000000 10000110 00001101 ' \
                 '00100010 10101110 00110000'
    expected = bits(expected_s)
    assert expected == buff.getbits()
    matrix = encoder.make_matrix(version)
    encoder.add_finder_patterns(matrix, is_micro=version < 1)
    encoder.add_codewords(matrix, buff, version=version)
    ref_matrix = read_matrix('iso-i3_code_placement')
    assert ref_matrix == matrix


def _make_figure22_matrix():
    version = consts.VERSION_M4
    matrix = encoder.make_matrix(version)
    for row in matrix:
        for i in range(len(row)):
            if row[i] == 0x2:
                row[i] = 0x0
    encoder.add_finder_patterns(matrix, True)
    return matrix


def test_figure22_mask0():
    # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
    # Figure 22 - Mask 0
    version = consts.VERSION_M4
    matrix = _make_figure22_matrix()
    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
                                                    proposed_mask=0)
    assert 0 == mask
    # Format info = dark modules
    for i in range(9):
        matrix[8][i] = 0x1
        matrix[i][8] = 0x1
    ref_matrix = read_matrix('fig-22-mask-0')
    assert len(ref_matrix) == len(matrix)
    assert ref_matrix == matrix


def test_figure22_mask1():
    # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
    # Figure 22 - Mask 1
    version = consts.VERSION_M4
    matrix = _make_figure22_matrix()
    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
                                                    proposed_mask=1)
    assert 1 == mask
    # Format info = dark modules
    for i in range(9):
        matrix[8][i] = 0x1
        matrix[i][8] = 0x1
    ref_matrix = read_matrix('fig-22-mask-1')
    assert len(ref_matrix) == len(matrix)
    assert ref_matrix == matrix


def test_figure22_mask2():
    # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
    # Figure 22 - Mask 2
    version = consts.VERSION_M4
    matrix = _make_figure22_matrix()
    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
                                                    proposed_mask=2)
    assert 2 == mask
    # Format info = dark modules
    for i in range(9):
        matrix[8][i] = 0x1
        matrix[i][8] = 0x1
    ref_matrix = read_matrix('fig-22-mask-2')
    assert len(ref_matrix) == len(matrix)
    assert ref_matrix == matrix


def test_figure22_mask3():
    # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
    # Figure 22 - Mask 3
    version = consts.VERSION_M4
    matrix = _make_figure22_matrix()
    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
                                                    proposed_mask=3)
    assert 3 == mask
    # Format info = dark modules
    for i in range(9):
        matrix[8][i] = 0x1
        matrix[i][8] = 0x1
    ref_matrix = read_matrix('fig-22-mask-3')
    assert len(ref_matrix) == len(matrix)
    assert ref_matrix == matrix


def test_fig23_best_mask():
    # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
    # Figure 23
    matrix = read_matrix('fig-23-unmasked')
    mask, matrix = encoder.find_and_apply_best_mask(matrix, 1, is_micro=False)
    assert 0 == mask
    ref_matrix = read_matrix('fig-23-mask-0')
    assert ref_matrix == matrix


def test_format_info_figure26():
    # 7.9.2 Micro QR Code symbols (page 57)
    version = consts.VERSION_M1
    mask = 3
    matrix = tuple([bytearray([0x0] * 11) for i in range(11)])
    encoder.add_timing_pattern(matrix, is_micro=True)
    encoder.add_finder_patterns(matrix, is_micro=True)
    encoder.add_format_info(matrix, version=version, error=None,
                            mask_pattern=mask)
    ref_matrix = read_matrix('fig-26')
    assert len(ref_matrix) == len(matrix)
    assert ref_matrix == matrix


if __name__ == '__main__':
    pytest.main([__file__])



# -*- encoding: utf-8 -*-
"""\
Create the benchmark charts.
"""
import os
import csv
from decimal import Decimal
import pygal


def create_chart(title, data, filename):
    chart = pygal.HorizontalBar(title=title, x_title='Time (ms)', js=(),
                                print_labels=True, show_legend=False)
    for name, val in data:
        chart.add(name, val)
    chart.render_to_file(filename)


def create_charts():
    color_map = {
        'PyQRCode': '#F44336',
        'PyQRCodeNG': '#b4750f',
        'qrcode': '#3F51B5',
        'qrcode path': '#3F51B5',
        'qrcode rects': '#26854F',
        'qrcodegen': '#009688',
        'Segno': '#FFC107',
    }
    create_1m_data = []
    create_7q_data = []
    create_30h_data = []
    svg_data = []
    png_data = []
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'out', 'results.csv')) as f:
        reader = csv.reader(f)
        for row in reader:
            name, val = row
            if name.endswith(' create 1-M'):
                create_1m_data.append((name.replace(' create 1-M', ''), val))
            elif name.endswith(' create 7-Q'):
                create_7q_data.append((name.replace(' create 7-Q', ''), val))
            elif name.endswith(' create 30-H'):
                create_30h_data.append((name.replace(' create 30-H', ''), val))
            elif name.endswith(' PNG 1-M'):
                png_data.append((name.replace(' PNG 1-M', ''), val))
            elif ' SVG' in name:
                svg_data.append((name.replace(' SVG', ''), val))
    output_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), '../docs/_static/')
    for data, title, filename in ((create_1m_data, 'Create a 1-M QR Code', os.path.join(output_dir, 'chart_create_1m.svg')),
                                  (create_7q_data, 'Create a 7-Q QR Code', os.path.join(output_dir, 'chart_create_7q.svg')),
                                  (create_30h_data, 'Create a 30-H QR Code', os.path.join(output_dir, 'chart_create_30h.svg')),
                                  (svg_data, 'Create a 1-M QR Code and write SVG', os.path.join(output_dir, 'chart_svg.svg')),
                                  (png_data, 'Create a 1-M QR Code and write PNG', os.path.join(output_dir, 'chart_png.svg'))):
        create_chart(title,
                     [(name, [{'value': float(val), 'color': color_map[name], 'label': name}]) for name, val in sorted(data, key=lambda t: Decimal(t[1]))],
                     filename)


if __name__ == '__main__':
    create_charts()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests if Segno produces the correct Covid certificate QR Code

See
* <https://github.com/Digitaler-Impfnachweis/covpass-android/issues/93>
* <https://github.com/lincolnloop/python-qrcode/issues/244>
"""
from __future__ import absolute_import
import segno
try:
    from .tutils import read_matrix
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from tutils import read_matrix


def test_covid_cert():
    cert_raw = "HC1:6BFOXN*TS0BI$ZD8UH76PCM7ZJ7GZ20II/P3 43JUB/EBUMIU/3%$C2L9KST--M:UC*GPXS40 LHZA KEW%G%9DJ6K1AD1WMN" \
               "+I0JK1WLB4DCHGLII3LN25OVLNF7LHCIZ09.9DM*GC9M$7JE9MZIIDOM1JAF.7IFNDXI03L9ZI4Q5%H0AN8EH06YB-TIP+P6OIB.V" \
               "T*QNC2BFUF$SU%BO*N5PIBPIAOI-+R2YBV44PZB6H0CJ0QCK0YQK*R3T3+7A.N88J4R$F/MAITH6QS03L0QIRR97I2HOAXL92L0. " \
               "KOKGTM8$M8SNCT64RR7+V4YC5/HQRPOHCR6W9.DPEA7IB65C94JB+ONS.CUIA7LE9JAF+B**O3-SY$NRUEG1AK/4C6DM.SY$NWYMG" \
               "3GUBRCEIFRMLNKNM8POCJPG8/0NOLZGIZCUOY55*LTUJS1D7.J9$7$.IWQOMJV$6M53AA:6Z2S./BXPHRKGJUHB1WN5LH/M0:3P+E" \
               "$$BWWNB8V5ZRF9JMKAX AAOL3H02YF*OF"
    # Don't use another error correction mode than "M", not sure if this is a requirement, though
    qr = segno.make(cert_raw, error='m', boost_error=False)
    assert 'M' == qr.error
    assert 15 == qr.version
    ref_matrix = read_matrix('covid-cert')
    assert ref_matrix
    assert ref_matrix == qr.matrix


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the command line script.
"""
from __future__ import absolute_import, unicode_literals
import os
import io
import tempfile
import gzip
import shutil
import xml.etree.ElementTree as etree
import pytest
from segno import cli


def test_defaults():
    args = cli.parse([''])
    assert args.content == ['']
    assert args.error is None
    assert args.mode is None
    assert args.pattern is None
    assert args.version is None
    assert args.scale == 1
    assert args.encoding is None
    assert not args.micro
    assert args.output is None
    assert args.border is None
    assert args.dark is None
    assert args.light is None
    assert args.boost_error
    assert not args.seq
    assert args.symbol_count is None
    # PNG
    assert not args.dpi
    assert args.finder_dark is None
    assert args.finder_light is None
    assert args.data_dark is None
    assert args.data_light is None
    assert args.alignment_dark is None
    assert args.alignment_light is None
    assert args.timing_dark is None
    assert args.timing_light is None
    assert args.format_dark is None
    assert args.format_light is None
    assert args.version_dark is None
    assert args.version_light is None
    assert args.quiet_zone is None
    assert args.dark_module is None
    assert args.separator is None
    # SVG
    assert args.xmldecl
    assert not args.no_classes
    assert args.svgencoding == 'utf-8'
    assert args.title is None
    assert args.desc is None
    assert args.svgns is True
    assert args.svgid is None
    assert args.svgclass is None
    assert args.lineclass is None
    assert args.omitsize is False
    assert args.unit is None
    assert args.svgversion is None
    assert args.nl is True
    assert args.draw_transparent is False
    # Terminal
    assert args.compact is False


def test_segno_version():
    try:
        cli.parse(['--ver', ''])
    except SystemExit as ex:
        assert 0 == ex.code


def test_segno_version_shortcut():
    try:
        cli.parse(['-V', ''])
    except SystemExit as ex:
        assert 0 == ex.code


def test_noargs():
    try:
        cli.parse([])
    except SystemExit as ex:
        assert 1 == ex.code


def test_error():
    args = cli.parse(['-e', 'm', ''])
    assert args.error == 'M'
    qr = cli.make_code(args)
    assert 'H' == qr.error


def test_error2():
    args = cli.parse(['-e', 'M', ''])
    assert args.error == 'M'
    qr = cli.make_code(args)
    assert 'H' == qr.error


def test_error3():
    args = cli.parse(['-e', '-', '123'])
    assert args.error is None
    qr = cli.make_code(args)
    assert not qr.is_micro
    assert 1 == qr.version
    assert 'H' == qr.error


def test_error_allow_micro():
    args = cli.parse(['-e', '-', '--micro', '123'])
    assert args.error is None
    qr = cli.make_code(args)
    assert qr.is_micro
    assert 'M1' == qr.version
    assert qr.error is None


def test_error4():
    args = cli.parse(['--error=q', '--no-error-boost', ''])
    assert args.error == 'Q'
    qr = cli.make_code(args)
    assert 'Q' == qr.error


def test_version():
    args = cli.parse(['-v', '1', ''])
    assert args.version == '1'
    qr = cli.make_code(args)
    assert 1 == qr.version


def test_version2():
    args = cli.parse(['--version', '40', ''])
    assert args.version == '40'
    qr = cli.make_code(args)
    assert 40 == qr.version


def test_version_micro():
    args = cli.parse(['-v', 'M1', '0'])
    assert args.version == 'M1'
    qr = cli.make_code(args)
    assert 'M1' == qr.version


def test_version_micro_m1():
    args = cli.parse(['-v', 'M1', '12345'])
    assert args.version == 'M1'
    qr = cli.make_code(args)
    assert 'M1' == qr.version


def test_version_micro_m1_automatic():
    args = cli.parse(['--micro', '12345'])
    qr = cli.make_code(args)
    assert 'M1' == qr.version


def test_version_micro_m2_automatic():
    args = cli.parse(['--micro', '123456'])
    qr = cli.make_code(args)
    assert 'M2' == qr.version


def test_mode():
    args = cli.parse(['-m', 'alphanumeric', 'A'])
    assert args.mode == 'alphanumeric'
    qr = cli.make_code(args)
    assert 'alphanumeric' == qr.mode


def test_mode2():
    args = cli.parse(['--mode=byte', ''])
    assert args.mode == 'byte'
    qr = cli.make_code(args)
    assert 'byte' == qr.mode


def test_pattern():
    args = cli.parse(['-p', '1', ''])
    assert args.pattern == 1
    qr = cli.make_code(args)
    assert qr.mask == 1


def test_pattern2():
    args = cli.parse(['--pattern', '5', ''])
    assert args.pattern == 5
    qr = cli.make_code(args)
    assert qr.mask == 5


def test_micro_false():
    args = cli.parse(['--no-micro', ''])
    assert not args.micro
    qr = cli.make_code(args)
    assert not qr.is_micro


def test_micro_true():
    args = cli.parse(['--micro', ''])
    assert args.micro
    qr = cli.make_code(args)
    assert qr.is_micro


def test_boost_error_disable():
    args = cli.parse(['--no-error-boost', ''])
    assert not args.boost_error


def test_border():
    args = cli.parse(['--border', '0', ''])
    assert args.border == 0


def test_border_shortcut():
    args = cli.parse(['-b', '10', ''])
    assert args.border == 10


def test_scale():
    args = cli.parse(['--scale=1.6', ''])
    assert args.scale == 1.6


def test_scale2():
    args = cli.parse(['--scale=2.0', ''])
    assert args.scale == 2
    assert isinstance(args.scale, int)


def test_scale_shortcut():
    args = cli.parse(['-s=1.6', ''])
    assert args.scale == 1.6


def test_sequence():
    args = cli.parse(['--seq', '-v=1', ''])
    assert args.seq
    assert '1' == args.version


def test_sequence_symbol_count():
    args = cli.parse(['--seq', '--symbol-count=4', ''])
    assert args.seq
    assert 4 == args.symbol_count


def test_sequence_symbol_count_shortcut():
    args = cli.parse(['--seq', '-sc=8', ''])
    assert args.seq
    assert 8 == args.symbol_count


def test_sequence_output():
    directory = tempfile.mkdtemp()
    assert 0 == len(os.listdir(directory))
    cli.main(['--seq', '-v=1', '-e=m', '-o=' + os.path.join(directory,
                                                            'test.svg'),
              'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'])
    number_of_files = len(os.listdir(directory))
    shutil.rmtree(directory)
    assert 4 == number_of_files


def test_dark():
    args = cli.parse(['--dark', 'green', ''])
    assert args.dark == 'green'
    assert cli.build_config(args)['dark'] == 'green'


@pytest.mark.parametrize('arg', ['transparent', 'trans'])
def test_dark_transparent(arg):
    args = cli.parse(['--dark={}'.format(arg), '-output=x.png', ''])
    assert args.dark == arg
    assert cli.build_config(args)['dark'] is None


def test_light():
    args = cli.parse(['--light', 'red', ''])
    assert args.light == 'red'


@pytest.mark.parametrize('arg', ['transparent', 'trans'])
def test_light_transparent(arg):
    args = cli.parse(['--light={}'.format(arg), '-output=x.png', ''])
    assert args.light == arg
    assert cli.build_config(args)['light'] is None


def test_error_code():
    try:
        cli.main(['--version=M1', '--seq', '"This is a test"'])
    except SystemExit as ex:
        assert 1 == ex.code


@pytest.mark.parametrize('arg', ['-o', '--output'])
@pytest.mark.parametrize('ext, expected, mode', [('svg', b'<?xml ', 'rb'),
                                                 ('pdf', b'%PDF-', 'rb'),
                                                 ('png', b'\211PNG\r\n\032\n', 'rb'),
                                                 ('svgz', b'\x1f\x8b\x08', 'rb'),
                                                 ('txt', '000000', 'rt'),
                                                 ('eps', '%!PS-Adobe-3.0 EPSF-3.0', 'rt'),
                                                 ('ans', '\033[7m       ', 'rt'),
                                                 ('pam', b'P7', 'rb'),
                                                 ('pbm', b'P4\n', 'rb'),
                                                 ('ppm', b'P6', 'rb'),
                                                 ('xbm', '#define ', 'rt'),
                                                 ('xpm', '/* XPM */', 'rt'),
                                                 ('tex', '% Creator: ', 'rt'),
                                                 ])
def test_output(arg, ext, expected, mode):
    f = tempfile.NamedTemporaryFile('w', suffix='.{0}'.format(ext),
                                    delete=False)
    f.close()
    try:
        cli.main(['test', arg, f.name])
        f = open(f.name, mode=mode)
        val = f.read(len(expected))
        f.close()
        assert expected == val
    finally:
        os.unlink(f.name)


def test_terminal(capsys):
    cli.main(['test'])
    out, err = capsys.readouterr()
    assert out
    assert '' == err
    assert '\033[0m' in out


def test_terminal_compact(capsys):
    cli.main(['--compact', 'test'])
    out, err = capsys.readouterr()
    assert out
    assert '' == err
    assert '\u2580' in out


# -- PNG
def test_dpi():
    args = cli.parse(['--dpi=300', ''])
    assert 300 == args.dpi


# -- SVG
def test_xmldecl():
    args = cli.parse(['--output=x.svg', ''])
    assert args.xmldecl
    assert cli.build_config(args)['xmldecl'] is True


def test_omit_xmldecl():
    args = cli.parse(['--no-xmldecl', '--output=x.svg', ''])
    assert not args.xmldecl
    assert cli.build_config(args)['xmldecl'] is False


def test_not_omit_classes():
    args = cli.parse(['--output=x.svg', ''])
    assert not args.no_classes
    config = cli.build_config(args)
    assert 'svgclass' not in config
    assert 'lineclass' not in config


def test_omit_classes():
    args = cli.parse(['--no-classes', '--output=x.svg', ''])
    assert args.no_classes
    config = cli.build_config(args)
    assert config['svgclass'] is None
    assert config['lineclass'] is None


def test_encoding():
    args = cli.parse(['--output=x.svg', ''])
    assert args.svgencoding == 'utf-8'
    assert cli.build_config(args)['encoding'] == 'utf-8'


def test_encoding2():
    args = cli.parse(['--svgencoding=ascii', '--output=x.svg', ''])
    assert args.svgencoding == 'ascii'
    assert cli.build_config(args)['encoding'] == 'ascii'


def test_encoding3():
    # Ignore --encoding since it is used to *create* a QR code
    args = cli.parse(['--encoding=latin1', '--output=x.svg', ''])
    assert args.svgencoding == 'utf-8'
    assert cli.build_config(args)['encoding'] == 'utf-8'


def test_title():
    args = cli.parse(['--output=x.svg', ''])
    assert args.title is None
    assert cli.build_config(args)['title'] is None


def test_title2():
    args = cli.parse(['--title=Magnolia', '--output=x.svg', ''])
    assert args.title == 'Magnolia'
    assert cli.build_config(args)['title'] == 'Magnolia'


def test_desc():
    args = cli.parse(['--output=x.svg', ''])
    assert args.desc is None
    assert cli.build_config(args)['desc'] is None


def test_desc2():
    args = cli.parse(['--desc=Magnolia', '--output=x.svg', ''])
    assert args.desc == 'Magnolia'
    assert cli.build_config(args)['desc'] == 'Magnolia'


def test_nl():
    args = cli.parse(['--output=x.svg', ''])
    assert args.nl is True
    assert cli.build_config(args)['nl'] is True


def test_nl2():
    args = cli.parse(['--no-newline', '--output=x.svg', ''])
    assert not args.nl
    assert cli.build_config(args)['nl'] is False


def test_ns():
    args = cli.parse(['--output=x.svg', ''])
    assert args.svgns is True
    assert cli.build_config(args)['svgns'] is True


def test_ns2():
    args = cli.parse(['--no-namespace', '--output=x.svg', ''])
    assert not args.svgns
    assert cli.build_config(args)['svgns'] is False


def test_svgid():
    args = cli.parse(['--output=x.svg', ''])
    assert args.svgid is None
    assert 'svgid' not in cli.build_config(args)


def test_svgid2():
    args = cli.parse(['--svgid=magnolia', '--output=x.svg', ''])
    assert args.svgid == 'magnolia'
    assert cli.build_config(args)['svgid'] == 'magnolia'


def test_svgclass():
    args = cli.parse(['--output=x.svg', ''])
    assert args.svgclass is None
    assert 'svgclass' not in cli.build_config(args)


def test_svgclass2():
    args = cli.parse(['--svgclass=magnolia', '--output=x.svg', ''])
    assert args.svgclass == 'magnolia'
    assert cli.build_config(args)['svgclass'] == 'magnolia'


_SVG_NS = 'http://www.w3.org/2000/svg'


def _parse_xml(buff):
    """\
    Parses XML and returns the root element.
    """
    buff.seek(0)
    return etree.parse(buff).getroot()


def test_svgclass3():
    fname = _make_tmp_svg_filename()
    res = cli.main(['--output={0}'.format(fname), 'test'])
    assert 0 == res
    with open(fname, 'rb') as f:
        data = io.BytesIO(f.read())
    root = _parse_xml(data)
    assert root is not None
    assert root.get('class') is not None
    res = cli.main(['--svgclass', '', '--output={0}'.format(fname), 'test'])
    assert 0 == res
    with open(fname, 'rb') as f:
        data = io.BytesIO(f.read())
    os.unlink(fname)
    root = _parse_xml(data)
    assert root is not None
    assert root.get('class') is None


def test_svg_lineclass():
    args = cli.parse(['--output=x.svg', ''])
    assert args.lineclass is None
    assert 'lineclass' not in cli.build_config(args)


def test_svg_lineclass2():
    args = cli.parse(['--lineclass=magnolia', ''])
    assert args.lineclass == 'magnolia'
    assert cli.build_config(args)['lineclass'] == 'magnolia'


def test_lineclass3():
    fname = _make_tmp_svg_filename()
    res = cli.main(['--output={0}'.format(fname), 'test'])
    assert 0 == res
    with open(fname, 'rb') as f:
        data = io.BytesIO(f.read())
    root = _parse_xml(data)
    assert root is not None
    path = root[0]
    assert path is not None
    assert path.get('class') is not None
    res = cli.main(['--lineclass', '', '--output={0}'.format(fname), 'test'])
    assert 0 == res
    with open(fname, 'rb') as f:
        data = io.BytesIO(f.read())
    os.unlink(fname)
    root = _parse_xml(data)
    path = root[0]
    assert path is not None
    assert path.get('class') is None


def test_omitsize():
    args = cli.parse(['--output=x.svg', ''])
    assert not args.omitsize
    assert cli.build_config(args)['omitsize'] is False


def test_omitsize2():
    args = cli.parse(['--no-size', ''])
    assert args.omitsize
    assert cli.build_config(args)['omitsize'] is True


def test_unit():
    args = cli.parse([''])
    assert args.unit is None
    assert cli.build_config(args)['unit'] is None


def test_unit2():
    args = cli.parse(['--unit=cm', ''])
    assert args.unit == 'cm'
    assert cli.build_config(args)['unit'] == 'cm'


def test_svgversion():
    args = cli.parse([''])
    assert args.svgversion is None
    assert cli.build_config(args)['svgversion'] is None


def test_svgversion2():
    args = cli.parse(['--svgversion=1', ''])
    assert args.svgversion == 1.0
    assert cli.build_config(args)['svgversion'] == 1.0


def test_svgversion3():
    args = cli.parse(['--svgversion=1.1', ''])
    assert args.svgversion == 1.1
    assert cli.build_config(args)['svgversion'] == 1.1


def test_draw_transparent():
    fname = _make_tmp_svg_filename()
    res = cli.main(['--dark=green', '--finder-dark=green', '--dark-module=blue',
                    '--align-light=yellow', '--quiet-zone=yellow',
                    '--output={0}'.format(fname), 'test'])
    assert 0 == res
    with open(fname, 'rb') as f:
        data = io.BytesIO(f.read())
    root = _parse_xml(data)
    paths = root.findall('.//{%s}path' % _SVG_NS)
    assert 3 == len(paths)
    res = cli.main(['--dark=green', '--finder-dark=green', '--dark-module=blue',
                    '--align-light=yellow', '--quiet-zone=yellow',
                    '--draw-transparent', '--output={0}'.format(fname), 'test'])
    assert 0 == res
    with open(fname, 'rb') as f:
        data = io.BytesIO(f.read())
    os.unlink(fname)
    root = _parse_xml(data)
    paths = root.findall('.//{%s}path' % _SVG_NS)
    assert 4 == len(paths)
    assert 1 == len([p for p in paths if p.attrib.get('stroke') is None])


def test_png_svg_command():
    args = cli.parse(['--svgversion=1.1', ''])
    assert args.svgversion == 1.1
    assert 'svgversion' in cli.build_config(args)
    assert 'svgversion' not in cli.build_config(args, filename='x.png')


def test_output_svgz():
    f = tempfile.NamedTemporaryFile('w', suffix='.svgz', delete=False)
    f.close()
    res = cli.main(['--scale=10', '--dark=red', '--output={0}'.format(f.name),
                    'test'])
    assert 0 == res
    with gzip.open(f.name) as f:
        content = f.read()
    os.unlink(f.name)
    assert b'scale(10)' in content
    assert b'stroke="red"' in content


def _make_tmp_svg_filename():
    f = tempfile.NamedTemporaryFile('w', suffix='.svg', delete=False)
    f.close()
    return f.name


if __name__ == '__main__':
    pytest.main([__file__])



from django import forms
from django.core.validators import RegexValidator


class TicketForm(forms.Form):
    """Simple form for the Ticket model."""
    name = forms.CharField(label='Name', max_length=150, min_length=1,
                           validators=[RegexValidator(r'^[A-Z0-9]+$')])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #18.
<https://github.com/heuer/segno/issues/18>
"""
from __future__ import unicode_literals, absolute_import
import segno


def test_issue_18():
    qr = segno.make_qr('')
    assert 1 == qr.version
    assert 'byte' == qr.mode
    assert 'H' == qr.error


def test_issue_18_micro():
    qr = segno.make_micro('')
    assert 'M3' == qr.version
    assert 'byte' == qr.mode
    assert 'M' == qr.error


def test_issue_18_automatic():
    qr = segno.make('')
    assert 'M3' == qr.version
    assert 'byte' == qr.mode
    assert 'M' == qr.error


def test_issue_18_zero():
    qr = segno.make_qr(0)
    assert 1 == qr.version
    assert 'numeric' == qr.mode
    assert 'H' == qr.error


def test_issue_18_zero_micro():
    qr = segno.make_micro(0)
    assert 'M1' == qr.version
    assert 'numeric' == qr.mode
    assert qr.error is None


def test_issue_18_zero_automatic():
    qr = segno.make(0)
    assert 'M1' == qr.version
    assert 'numeric' == qr.mode
    assert qr.error is None


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue <https://github.com/pyqrcode/pyqrcodeNG/pull/13/>.

The initial test was created by Mathieu <https://github.com/albatros69>,
see the above mentioned pull request.

Adapted for Segno to check if it suffers from the same problem.
"""
from __future__ import absolute_import, unicode_literals
import segno


def test_autodetect():
    data = 'Émetteur'
    qr = segno.make(data)
    assert qr.mode == 'byte'


def test_encoding():
    encoding = 'iso-8859-15'
    data = 'Émetteur'
    qr = segno.make(data.encode(encoding))
    assert qr.mode == 'byte'
    qr2 = segno.make(data, encoding=encoding)
    assert qr2 == qr


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests if the PNG serializer does not add more colors than needed.

See also issue <https://github.com/heuer/segno/issues/62>
"""
from __future__ import unicode_literals, absolute_import
import io
import pytest
import segno


def test_plte():
    qr = segno.make_qr('test')
    assert qr.version < 7
    dark = 'red'
    buff_1 = io.BytesIO()
    buff_2 = io.BytesIO()
    qr.save(buff_1, kind='png', dark=dark, finder_dark=dark, version_dark='green')
    qr.save(buff_2, kind='png', dark=dark)
    assert buff_1.getvalue() == buff_2.getvalue()


def test_plte2():
    qr = segno.make_qr('test')
    assert qr.version < 7
    dark = 'red'
    buff_1 = io.BytesIO()
    buff_2 = io.BytesIO()
    qr.save(buff_1, kind='png', dark=dark, finder_dark=dark, version_dark='green')
    qr.save(buff_2, kind='png', dark=dark)
    assert buff_1.getvalue() == buff_2.getvalue()


def test_plte3():
    qr = segno.make_qr('test')
    assert qr.version < 7
    dark = 'red'
    buff_1 = io.BytesIO()
    buff_2 = io.BytesIO()
    qr.save(buff_1, kind='png', dark=dark, finder_dark=dark, version_dark='green')
    qr.save(buff_2, kind='png', dark=dark)
    assert buff_1.getvalue() == buff_2.getvalue()


def test_plte_micro():
    qr = segno.make_micro('RAIN')
    dark = 'red'
    buff_1 = io.BytesIO()
    buff_2 = io.BytesIO()
    qr.save(buff_1, kind='png', dark=dark, finder_dark=dark, alignment_dark='green')
    qr.save(buff_2, kind='png', dark=dark)
    assert buff_1.getvalue() == buff_2.getvalue()


def test_plte_micro2():
    qr = segno.make_micro('RAIN')
    dark = 'red'
    buff_1 = io.BytesIO()
    buff_2 = io.BytesIO()
    qr.save(buff_1, kind='png', dark=dark, finder_dark=dark, dark_module='green')
    qr.save(buff_2, kind='png', dark=dark)
    assert buff_1.getvalue() == buff_2.getvalue()


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue 71
<https://github.com/heuer/segno/issues/71>
"""
from __future__ import absolute_import, unicode_literals
import pytest
from segno import consts, writers


def test_issue71():
    cm = writers._make_colormap(7, dark='blue', light='white', finder_dark=None, finder_light=None)
    assert cm[consts.TYPE_FINDER_PATTERN_DARK] is None
    assert cm[consts.TYPE_FINDER_PATTERN_LIGHT] is None


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Utility functions for test cases.
"""
import os
import io


def matrix_looks_valid(matrix):
    """\
    Returns if the matrix contains just ``0x0`` and ``0x1`` values.
    Does not check if the the matrix represents a valid (Micro) QR Code.

    :param matrix: tuple of bytearrays
    :return:
    """
    invalid_values = []
    for i, row in enumerate(matrix):
        for j, bit in enumerate(row):
            if bit not in (0x0, 0x1):
                invalid_values.append((i, j, bit))
    if not invalid_values:
        return True, ''
    msg = 'Invalid values: '
    for i, j, bit in invalid_values:
        msg += '\nrow: {0}, col {1}, value: {2}'.format(i, j, bit)
    return False, msg


def read_matrix(name):
    """\
    Helper function to read a matrix from /ref_matrix. The file extension .txt
    is added automatically.

    :return: A tuple of bytearrays
    """
    matrix = []
    with io.open(os.path.join(os.path.dirname(__file__), 'ref_matrix/{0}.txt'.format(name)), 'rt') as f:
        for row in f:
            matrix.append(bytearray([int(i) for i in row if i != '\n']))
    return tuple(matrix)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
PNG related tests.
"""
from __future__ import unicode_literals, absolute_import
import io
import os
import re
import pytest
import segno
from png import Reader as PNGReader


def test_hexcolor_too_short():
    qr = segno.make_qr('test')
    with pytest.raises(ValueError):
        qr.save(io.BytesIO(), kind='png', dark='#FFFFF')


def test_hexcolor_too_short_background():
    qr = segno.make_qr('test')
    with pytest.raises(ValueError):
        qr.save(io.BytesIO(), kind='png', light='#FFFFF')


def test_hexcolor_too_long():
    qr = segno.make_qr('test')
    with pytest.raises(ValueError):
        qr.save(io.BytesIO(), kind='png', dark='#0000000')


def test_hexcolor_too_long_background():
    qr = segno.make_qr('test')
    with pytest.raises(ValueError):
        qr.save(io.BytesIO(), kind='png', light='#0000000')


def test_dark_eq_light():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='#000', light='#000')
    assert out.getvalue()


_has_palette = re.compile(br'PLTE').search
_has_transparency = re.compile(br'tRNS').search


def test_greyscale():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='#000', light='#fff')
    assert not _has_palette(out.getvalue())
    assert not _has_transparency(out.getvalue())


def test_greyscale2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='white', light='black')
    assert not _has_palette(out.getvalue())
    assert not _has_transparency(out.getvalue())


def test_greyscale_trans():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='#000', light=None)
    assert not _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_greyscale_trans2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark=None, light='white')
    assert not _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_color():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='blue', light='white')
    assert _has_palette(out.getvalue())
    assert not _has_transparency(out.getvalue())


def test_color_trans():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='blue', light=None)
    assert _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_color_trans2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark=None, light='green')
    assert _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_color_rgba():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='#0000ffcc', light='white')
    assert _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_color_rgba2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='#000', light='#0000ffcc')
    assert _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_color_rgba_and_trans():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark='#0000ffcc', light=None)
    assert _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_color_rgba_and_trans2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dark=None, light='#0000ffcc')
    assert _has_palette(out.getvalue())
    assert _has_transparency(out.getvalue())


def test_scale():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    scale = 12
    width, height = qr.symbol_size(scale=scale)
    qr.save(out, kind='png', scale=12)
    out.seek(0)
    png_width, png_height, matrix = _get_png_info(file=out)
    assert (width, height) == (png_width, png_height)


def test_nodpi():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png')
    out.seek(0)
    assert b'pHYs' not in out.getvalue()


def test_nodpi_zero():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dpi=0)
    out.seek(0)
    assert b'pHYs' not in out.getvalue()


def test_dpi_negative():
    qr = segno.make('test')
    out = io.BytesIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='png', dpi=-3)


def test_dpi():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png', dpi=300)
    out.seek(0)
    assert b'pHYs' in out.getvalue()
    # pHYs 11811 (11811 meters = 300 dpi / 0.0254)
    assert b'\x70\x48\x59\x73\x00\x00\x2E\x23\x00\x00\x2E\x23\x01\x78\xA5\x3F\x76' in out.getvalue()


def png_as_matrix(buff, border):
    """\
    Reads the PNG from the provided buffer and returns the code matrix (list
    of lists containing 0 .. 1 values).
    """
    buff.seek(0)
    w, h, pixels = _get_png_info(file=buff)
    # PNG: white = 1, black = 0. QR code: white = 0, black = 1
    from_idx, to_idx = border, -border
    if border == 0:
        from_idx = 0
        to_idx = len(pixels)
    res = []
    for row in pixels[from_idx:to_idx]:
        res.append([bit ^ 0x1 for bit in row[from_idx:to_idx]])
    return res


def _make_pixel_array(pixels, is_greyscale):
    """\
    Returns a list of lists. Each list contains 0 and/or 1.
    0 == black, 1 == white.

    :param bool is_greyscale: Indiciates if this function must convert RGB colors
            into black/white (supported values: (0, 0, 0) = black and
            (255, 255, 255) = white)
    """
    def bw_color(r, g, b):
        rgb = r, g, b
        if rgb == (0, 0, 0):
            return 0
        elif rgb == (255, 255, 255):
            return 1
        else:
            raise ValueError('Unexpected RGB tuple: {0})'.format(rgb))
    res = []
    if is_greyscale:
        res = [list(row[:]) for row in pixels]
    else:
        for row in pixels:
            it = [iter(row)] * 3
            res.append([bw_color(r, g, b) for r, g, b in zip(*it)])
    return res


def _get_reference_filename(filename):
    """\
    Returns an absolute path to the "reference" filename.
    """
    return os.path.join(os.path.dirname(__file__), 'ref/{0}'.format(filename))


def _get_png_info(**kw):
    """\
    Returns the width, height and the pixels of the provided PNG file.
    """
    reader = PNGReader(**kw)
    w, h, pixels, meta = reader.asDirect()
    return w, h, _make_pixel_array(pixels, meta['greyscale'])


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #4.
<https://github.com/heuer/segno/issues/4>
"""
from __future__ import unicode_literals, absolute_import
from segno import consts, encoder


def test_issue_4():
    qr = encoder.encode(0)
    assert consts.VERSION_M1 == qr.version
    assert qr.error is None


def test_issue_4_autodetect_micro():
    qr = encoder.encode(1)
    assert consts.VERSION_M1 == qr.version
    assert qr.error is None


def test_issue_4_explicit_error():
    qr = encoder.encode(1, error=None)
    assert consts.VERSION_M1 == qr.version
    assert qr.error is None


def test_issue_4_explicit_error2():
    qr = encoder.encode(1, error='m')
    assert consts.VERSION_M2 == qr.version
    assert consts.ERROR_LEVEL_M == qr.error


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #65.
<https://github.com/heuer/segno/issues/65>
"""
from __future__ import unicode_literals, absolute_import
import io
import base64
import pytest
import segno
try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote


def test_png_colorful():
    dark = 'darkred'
    data_dark = 'darkorange'
    data_light = 'yellow'
    qr = segno.make('Penny Lane', error='h')
    out = io.BytesIO()
    qr.save(out, 'png', dark=dark, data_dark=data_dark, data_light=data_light)
    data_uri = qr.png_data_uri(dark=dark, data_dark=data_dark,
                               data_light=data_light)
    assert data_uri
    d = base64.b64decode(data_uri[len('data:image/png;base64,'):])
    assert out.getvalue() == d


def test_svg_colorful():
    from segno.writers import _replace_quotes as replace_quotes
    dark = 'darkred'
    data_dark = 'darkorange'
    data_light = 'yellow'
    qr = segno.make('Penny Lane', error='h')
    out = io.BytesIO()
    qr.save(out, 'svg', xmldecl=False, nl=False, dark=dark, data_dark=data_dark,
            data_light=data_light)
    data_uri = qr.svg_data_uri(dark=dark, data_dark=data_dark,
                               data_light=data_light)
    assert data_uri
    d = data_uri[len('data:image/svg+xml;charset=utf-8,'):]
    assert quote(replace_quotes(out.getvalue()), safe=b'') == d


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2021 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Nox test runner configuration.
"""
import os
import sys
import re
from functools import partial
from itertools import chain
import shutil
import nox

_PY_VERSIONS = ('2.7', '3.7', '3.8', '3.9', '3.10', 'pypy', 'pypy3')
_PY_DEFAULT_VERSION = '3.9'

nox.options.sessions = ['test-2.7', 'test-{}'.format(_PY_DEFAULT_VERSION), 'test-pypy', 'test-pypy3']


@nox.session(python=_PY_VERSIONS)
def test(session):
    """\
    Run test suite.
    """
    if session.python == 'pypy':
        # See <https://github.com/heuer/segno/issues/80>
        pip_ver = session.run('pip', '--version', silent=True)
        m = re.search(r'pip ([0-9.]{4})', pip_ver)
        assert m
        if m.group(1) in ('20.2', '20.3'):
            session.run('pip', 'uninstall', '-y', 'pip')
            session.run('easy_install', 'pip==20.1.1')
    session.install('-Ur', 'requirements-testing.txt')
    session.install('.')
    session.run('py.test')


@nox.session(python=_PY_DEFAULT_VERSION)
def docs(session):
    """\
    Build the documentation.
    """
    session.install('-Ur', 'requirements-rtd.txt')
    output_dir = os.path.abspath(os.path.join(session.create_tmp(), 'output'))
    shutil.rmtree(output_dir, ignore_errors=True)
    doctrees, html, man = map(partial(os.path.join, output_dir), ['doctrees', 'html', 'man'])
    session.install('.')
    session.cd('docs')
    sphinx_build = partial(session.run, 'sphinx-build', '-W', '-d', doctrees, '.')
    sphinx_build('-b', 'html', html)
    sphinx_build('-b', 'man', man)
    sys.path.insert(0, os.path.abspath('..'))
    import segno
    if 'dev' not in segno.__version__:
        shutil.copyfile(os.path.join(man, 'segno.1'),
                        os.path.abspath(os.path.join(os.path.dirname(__file__), 'man', 'segno.1')))
        session.log("'man/segno.1' has been modified, don't forget to commit")


@nox.session(python=_PY_DEFAULT_VERSION)
def coverage(session):
    """\
    Run coverage.
    """
    session.install('coverage', '-Ur', 'requirements-testing.txt')
    session.install('.')
    output_dir = os.path.abspath(os.path.join(session.create_tmp(), 'html'))
    cover = partial(session.run, 'coverage')
    cover('erase')
    cover('run', './tests/alltests.py')
    cover('report')
    cover('html', '-d', output_dir)


@nox.session(python=_PY_DEFAULT_VERSION)
def lint(session):
    """\
    Run flake8
    """
    session.install('flake8', 'mypy')
    session.run('mypy', 'segno')
    session.run('flake8', 'segno')
    session.run('flake8', 'tests/')


@nox.session(python=_PY_DEFAULT_VERSION)
def benchmarks(session):
    """\
    Run the benchmarks and create the charts.
    """
    session.install('qrcode', 'Pillow', 'pyqrcode', 'pypng', 'qrcodegen', 'pygal')
    session.install('.')
    sandbox_dir = os.path.join(os.path.dirname(__file__), 'sandbox')
    session.run('python', os.path.join(sandbox_dir, 'benchmarks.py'))
    session.run('python', os.path.join(sandbox_dir, 'make_charts.py'))


#
# Release related tasks
# 1. nox -e start-release -- version-number
# 2. run tests, update docs, update changes
# 3. nox -e finish-release -- version-number
# 4. git push / git push origin --tags
# 5. nox -e build-release -- version-number
# 6. nox -e upload-release
#


@nox.session(name='start-release', python=_PY_DEFAULT_VERSION)
def start_release(session):
    """\
    Prepares a release.

    * Creates a new branch release-VERSION_NUMBER
    * Changes the version number in segno.__version__ to VERSION_NUMBER
    """
    session.install('packaging')
    git = partial(session.run, 'git', external=True)
    git('checkout', 'master')
    prev_version = _get_current_version(session)
    version = _validate_version(session)
    valid_version = bool(int(session.run('python', '-c', 'from packaging.version import parse;'
                                                         'prev_version = parse("{0}");'
                                                         'next_version = parse("{1}");'
                                                         'print(1 if prev_version < next_version else 0)'
                                         .format(prev_version, version), silent=True)))
    if not valid_version:
        session.error('Invalid version')
    release_branch = 'release-{}'.format(version)
    git('checkout', '-b', release_branch, 'master')
    _change_version(session, prev_version, version)
    git('add', 'segno/__init__.py')
    session.log('Now on branch "{}". Run the tests, run nox -e docs. Update CHANGES'.format(release_branch))
    session.log('When done, call nox -e finish-release -- {}'.format(version))


@nox.session(name='finish-release', python=_PY_DEFAULT_VERSION)
def finish_release(session):
    """\
    Finishes the release.

    * Merges the branch release-VERSION_NUMBER into master
    * Creates a tag VERSION_NUMBER
    * Increments the development version
    """
    version = _validate_version(session)
    release_branch = 'release-{}'.format(version)
    git = partial(session.run, 'git', external=True)
    git('checkout', 'master')
    git('merge', '--no-ff', release_branch, '-m', 'Merge release branch {}'.format(release_branch))
    git('tag', '-a', version, '-m', 'Release {}'.format(version))
    git('branch', '-d', release_branch)
    version_parts = version.split('.')
    patch = str(int(version_parts[2]) + 1)
    next_version = '.'.join(chain(version_parts[:2], patch)) + '.dev'
    _change_version(session, version, next_version)
    git('add', 'segno/__init__.py')
    git('commit', '-m', 'Incremented development version')
    session.log('Finished. Run git push / git push origin --tags and '
                'nox -e build-release -- {} / nox -e upload-release'.format(version))


@nox.session(name='build-release', python=_PY_DEFAULT_VERSION)
def build_release(session):
    """\
    Builds a release: Creates sdist and wheel
    """
    version = _validate_version(session)
    session.install('build', 'wheel')
    git = partial(session.run, 'git', external=True)
    git('fetch')
    git('fetch', '--tags')
    git('checkout', version)
    shutil.rmtree('dist', ignore_errors=True)
    session.run('python', '-m', 'build', '--no-isolation')
    git('checkout', 'master')


@nox.session(name='upload-release', python=_PY_DEFAULT_VERSION)
def upload_release(session):
    """\
    Uploads a release to PyPI
    """
    session.install('twine')
    twine = partial(session.run, 'twine')
    twine('check', 'dist/*')
    twine('upload', 'dist/*')


def _validate_version(session):
    if not session.posargs:
        session.error('No release version provided')
    elif len(session.posargs) > 1:
        session.error('Too many arguments')
    version = session.posargs[0]
    if not re.match(r'^[0-9]+\.[0-9]+\.[0-9]+$', version):
        session.error('Invalid version number: "{}"'.format(version))
    return version


def _get_current_version(session):
    """\
    Returns the current Segno version.
    """
    fn = os.path.abspath(os.path.join(os.path.dirname(__file__), 'segno/__init__.py'))
    with open(fn, 'r', encoding='utf-8') as f:
        content = f.read()
    m = re.search(r'^__version__ = ["\']([^"\']+)["\']$', content, flags=re.MULTILINE)
    if m:
        return m.group(1)
    session.error('Cannot find any version information')


def _change_version(session, previous_version, next_version):
    """\
    Changes the segno.__init__.__version__ from previous_version to next_version.
    """
    fn = os.path.abspath(os.path.join(os.path.dirname(__file__), 'segno/__init__.py'))
    with open(fn, 'r', encoding='utf-8') as f:
        content = f.read()
    new_content = re.sub(r'^(__version__ = ["\'])({0})(["\'])$'
                         .format(re.escape(previous_version)),
                         r'\g<1>{}\g<3>'.format(next_version), content,
                         flags=re.MULTILINE)
    if content != new_content:
        with open(fn, 'w', encoding='utf-8') as f:
            f.write(new_content)
        return
    session.error('Cannot modify version. Provided: "{}" (previous) "{}" (next)'
                  .format(previous_version, next_version))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Example how to add additional text to a QR code.

See <https://github.com/lincolnloop/python-qrcode/issues/199>
"""
import io
import os
from functools import partial
from PIL import Image, ImageColor, ImageDraw, ImageFont
import segno


def qr_with_text(qrcode: segno.QRCode, *, text: str = None,
                 font_path: str = None, font_size: int = 12, font_color=False,
                 line_spacing: int = None, scale: int = 1, border: int = None,
                 dark='#000', light='#fff', finder_dark=False, finder_light=False,
                 data_dark=False, data_light=False, version_dark=False,
                 version_light=False, format_dark=False, format_light=False,
                 alignment_dark=False, alignment_light=False, timing_dark=False,
                 timing_light=False, separator=False, dark_module=False,
                 quiet_zone=False) -> Image:
    """\
    Adds a text (i.e. the content of the QR code) to the QR code.

    See `segno.QRCode.save <https://segno.readthedocs.io/en/latest/api.html#segno.QRCode.save>`_
    and `PNG <https://segno.readthedocs.io/en/latest/api.html#png>`_ for a detailed
    description of the parameters.

    :param segno.QRCode qrcode: The QR code.
    :param str text: The text to add to the QR code.
    :param str font_path: Path to the font
    :param int font_size: Font size.
    :param font_color: See dark for valid values.
    :param int line_spacing: Spacing between the lines. If not provided, it
            is the half of the font size.
    :param scale: The scale.
    :param int border: Number indicating the size of the quiet zone.
            If set to ``None`` (default), the recommended border size
            will be used (``4`` for QR Codes, ``2`` for Micro QR Codes).
    :param dark: Color of the dark modules (default: black). The
            color can be provided as ``(R, G, B)`` tuple, as hexadecimal
            format (``#RGB``, ``#RRGGBB`` ``RRGGBBAA``), or web color
            name (i.e. ``red``).
    :param light: Color of the light modules (default: white).
            See `color` for valid values. If light is set to ``None`` the
            light modules will be transparent.
    :param finder_dark: Color of the dark finder modules (default: same as ``dark``)
    :param finder_light: Color of the light finder modules (default: same as ``light``)
    :param data_dark: Color of the dark data modules (default: same as ``dark``)
    :param data_light: Color of the light data modules (default: same as ``light``)
    :param version_dark: Color of the dark version modules (default: same as ``dark``)
    :param version_light: Color of the light version modules (default: same as ``light``)
    :param format_dark: Color of the dark format modules (default: same as ``dark``)
    :param format_light: Color of the light format modules (default: same as ``light``)
    :param alignment_dark: Color of the dark alignment modules (default: same as ``dark``)
    :param alignment_light: Color of the light alignment modules (default: same as ``light``)
    :param timing_dark: Color of the dark timing pattern modules (default: same as ``dark``)
    :param timing_light: Color of the light timing pattern modules (default: same as ``light``)
    :param separator: Color of the separator (default: same as ``light``)
    :param dark_module: Color of the dark module (default: same as ``dark``)
    :param quiet_zone: Color of the quiet zone modules (default: same as ``light``)
    """
    png = io.BytesIO()
    qrcode.save(png, kind='png', scale=scale, border=border, dark=dark,
                light=light, finder_dark=finder_dark, finder_light=finder_light,
                data_dark=data_dark, data_light=data_light, version_dark=version_dark,
                version_light=version_light, format_dark=format_dark,
                format_light=format_light, alignment_dark=alignment_dark,
                alignment_light=alignment_light, timing_dark=timing_dark,
                timing_light=timing_light, separator=separator,
                dark_module=dark_module, quiet_zone=quiet_zone)
    png.seek(0)
    img = Image.open(png)
    if text is None:
        return img
    font_path = font_path or os.path.join(os.path.dirname(__file__), 'font', 'DejaVuSansMono.ttf')
    font = ImageFont.truetype(font_path, font_size)
    width, height = img.size
    x, y = scale * (border if border is not None else qrcode.default_border_size), height
    line_spacing = line_spacing or font_size // 2
    lines = text.splitlines()
    # Calculate the additional space required for the text
    for line in lines:
        fw, fh = font.getsize(line)
        if fw > width:
            width = fw + font_size
        height += fh + line_spacing
    has_palette = img.mode == 'P'
    if has_palette:
        # The palette of the resulting image may be different from the
        # palette of the original image. To avoid problems with pasting the
        # QR code image into the resulting image, convert the QR code image to RGBA
        # This operation is reverted after drawing the text
        img = img.convert('RGBA')
    res_img = Image.new(img.mode, (width, height), color=quiet_zone or light)
    res_img.paste(img)
    draw = ImageDraw.Draw(res_img)
    font_color = font_color or dark
    draw_text = partial(draw.text, font=font, fill=ImageColor.getcolor(font_color, img.mode))
    for line in lines:
        draw_text((x, y), line)
        y += font_size + line_spacing
    if has_palette:
        res_img = res_img.convert('P')
    return res_img


if __name__ == '__main__':
    content = 'I read the news today, oh boy\n' \
              'About a lucky man who made the grade\n' \
              'And though the news was rather sad\n' \
              'Well, I just had to laugh\n' \
              'I saw the photograph'
    qr = segno.make(content)
    qr_with_text(qr, text=content, scale=6, dark='darkblue', light='#ffffb2',
                 quiet_zone='#eee').save('a-day-in-the-life-1.png')
    qr_with_text(qr, text=content, scale=6, dark='darkblue', quiet_zone='#eee',
                 font_color='darkgreen').save('a-day-in-the-life-2.png')
    qr_with_text(qr, text=content, scale=6).save('a-day-in-the-life-3.png')
    qr_with_text(qr, text=content, scale=6, font_size=32).save('a-day-in-the-life-4.png')
    qr_with_text(qr, text=content).save('a-day-in-the-life-5.png')
    qr_with_text(qr, text=content, border=0, scale=3).save('a-day-in-the-life-6.png')



# -*- coding: utf-8 -*-
# (c) Martijn van Rheenen
# BSD License
"""\
Tests against <https://github.com/mnooner256/pyqrcode/issues/50>

This test was created by Martijn van Rheenen <https://github.com/rheenen>
for PyQRCode. Adapted to Segno to check if it has the same problem.
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno


class FakeString(str):
    """\
    Create a mock class that *acts* like a string as far as needed for the
    QRCode constructor, but raises an exception in case shiftjis encoding is
    used on its value.

    This mimics the behaviour of Python on an environment where this codec is
    not installed.
    """
    def __new__(cls, *args, **kw):
        return str.__new__(cls, *args, **kw)

    def encode(self, encoding=None, errors='strict'):
        if encoding == 'shiftjis':
            raise LookupError("unknown encoding: shiftjis")
        return super(FakeString, self).encode(encoding, errors)


def test_constructing_without_shiftjis_encoding_available():
    content = FakeString("t123456789")
    code = segno.make(content, error="Q")
    assert 'byte' == code.mode


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue <https://github.com/mnooner256/pyqrcode/issues/17#issuecomment-419731805>
and <https://github.com/heuer/pyqrcode/issues/10>
"""
from __future__ import unicode_literals, absolute_import
import segno


def test_issue_10_17():
    qr = segno.make_qr('John’s Pizza')
    assert 1 == qr.version
    assert 'byte' == qr.mode
    assert 'M' == qr.error


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #23.
<https://github.com/heuer/segno/issues/23>
"""
from __future__ import unicode_literals, absolute_import
import segno


def test_boost_error_automatic():
    qr = segno.make('http://www.example.org/')
    assert '2-M' == qr.designator


def test_boost_error_disabled():
    qr = segno.make('http://www.example.org/', error='q', boost_error=False)
    assert '3-Q' == qr.designator


def test_boost_error_automatic2():
    qr = segno.make('http://www.example.org/', error='q')
    assert '3-H' == qr.designator


def test_boost_error_disabled2():
    qr = segno.make('http://www.example.org/', error='l', boost_error=False)
    assert '2-L' == qr.designator


def test_boost_error_disabled3():
    qr = segno.make('http://www.example.org/', error='h')
    assert '3-H' == qr.designator


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])
